

#define class
setClass("normalMixtureModel",representation = representation(p="numeric",mu="matrix",
                                                              sigma="array",dimension="numeric"))


#plot weighted single components of a ONE-DIMENSIONAL normal mixture
plotComponents = function(object,add=FALSE,main=""){
   
  if(object@dimension>1) return("Function not available!")
  if(object@dimension==1){
  
  #calculate needed xAxis
  getAxis = function(object){
    
    m=length(object@mu)
    param=c(object@p[1:(m-1)],object@mu,object@sigma)
    
    quantilesMin = NULL
    quantilesMax = NULL
    
    for(i in 1:m){ 
      quantilesMin = c(quantilesMin,qnorm(p=0.0001,mean=object@mu[i],sd=object@sigma[i]))
      quantilesMax = c(quantilesMax,qnorm(p=0.9999,mean=object@mu[i],sd=object@sigma[i]))                
    }
    
    return(c(min(quantilesMin),max(quantilesMax)))
  }
  xlim = getAxis(object)
  
  #calculate needed yAxis
  getYAxis = function(object){
    
    m=length(object@mu)
    param=c(object@p[1:(m-1)],object@mu,object@sigma)
    
    heightValues = numeric(m)
    
    for(i in 1:m){ 
      heightValues[i] = object@p[i] * dnorm(object@mu[i],mean=object@mu[i],sd=object@sigma[i])
    }
    return(max(heightValues))
    
  }
  ylim = c(0,getYAxis(object)*1.05)
  
  m=length(object@mu)
  param=c(object@p[1:(m-1)],object@mu,object@sigma)
  k = (length(param)+1)/3
  if(k==1){
    stop()
  }
  p   = param[1:(k-1)]
  p[k]= 1-sum(p)
  mu  = param[k:(2*k-1)]
  sig = param[(2*k):(3*k-1)]
  curve(p[1]*dnorm(x,mean=mu[1],sd=sig[1]),add=add,from=xlim[1],to=xlim[2],ylim=ylim,main=main,n=1000,ylab="component densities")
  y=numeric(1)
  for(i in 2:k)
    y= curve(p[i]*dnorm(x,mean=mu[i],sd=sig[i]),add=T,from=xlim[1],to=xlim[2],col=i,n=1000)
  }
}


#generic function to simulate a normal mixture
setMethod("simulate",signature(object="normalMixtureModel",nsim="numeric"),
          function(object,nsim,...){
            
            simuliere_daten <- function(anzahl, mu_vek, sigma_vek, gewichte, dimension = 2)
            {
              model <- "gaussmix"
              #Simuliere anzahl Punkte aus der Mischung
              stelle <- 1
              satz <- array(dim=c(anzahl,dimension))
              #wahre_cluster <- numeric(1000)
              while(stelle <= anzahl){
                komponente <- rmultinom(1,1,gewichte)#Whle zufllig eine Komponente aus
                komponente <- which(komponente == 1)
                
                if(model == "gaussmix")
                {
                  if(dimension > 1)
                  {
                    satz[stelle,] <- rmvnorm(1, mean = mu_vek[,komponente], sigma = sigma_vek[,,komponente])
                  }
                  
                  else
                  {
                    satz[stelle,] <- rnorm(1, mean = mu_vek[,komponente], sd = sigma_vek[,,komponente])
                  } 
                }
                
                
                
                else if(model=="tmix"){satz[,stelle] <- rmvt(1, sigma_vek[,,komponente], df = freiheitsgrade[komponente]) +  mu_vek[,komponente]}
                else if(model == "scewgauss"){satz[,stelle] <- rmsn(n = 1, xi = mu_vek[,komponente], Omega= sigma_vek[,,komponente], alpha = scewness[,komponente])}
                else return(0)
                
                stelle = stelle + 1
              }
              
              #satz <- data.frame(K1 = satz[1,], K2 = satz[2,] )
              return(satz)
            }
            
              
              return(simuliere_daten(
              anzahl=nsim, mu_vek=object@mu, sigma_vek=object@sigma, 
              gewichte=c(object@p,1-sum(object@p)), dimension = object@dimension))
              
            
          })


#generic function to simulate a normal mixture
gaussianMixtureMLE = 
                   function(x,
			    k,
			    object,
			    penP=0,
			    penSig=0,
			    doFI=FALSE,
			    tol_eps= 1e-11,
			    tol_delta = 1e-11,
			    tol_grad = 1e-11,
          tol_rlc = 1e-8,
			    tol_em = 1e-6,
			    verbose=FALSE,
			    bcl = 35,
			    maxit=10,
			    hard_conv = FALSE,
			    parallel = TRUE){
                       
            if(length(x[1,])==1) {x=matrix(x)}

            #one-dimensional fitting
            if(length(x[1,]) == 1){
              
              #estimating penalized MLE of a ONE-DIMENSIONAL normal mixture model
              fitPenMLEONE = function(x,k,p=NULL,mu=NULL,sigma=NULL,penaltyP=0,penaltySigma=0){
                
                if(is.null(p) || is.null(mu) || is.null(sigma)){
                  mm = Mclust(x,G=k,modelNames="V")
                  if(is.null(p)) p = mm$parameters$pro[1:(k-1)]
                  if(is.null(mu)) mu = mm$parameters$mean
                  if(is.null(sigma)) sigma = sqrt(mm$parameters$variance$scale)
                }
                
                #m=length(mu); if(length(sigma)!=m) return(warning("Length of mu and sigma do not match."))
                startParam = c(p,mu,sigma)
                
                #internal function for mleFitting
                mle_fitting = function(dataset,param,k,cp,csig){  
                  if(k==1){
                    erg.hyp=list(par=c(mean(dataset),sd(dataset)),value=-sum(log(dnorm(dataset,mean=mean(dataset),sd=sd(dataset)))))
                    
                    erg.hyp[[length(erg.hyp)+1]] = 2*erg.hyp$value + 2*2
                    names(erg.hyp)[length(erg.hyp)]="aic"
                    
                    erg.hyp[[length(erg.hyp)+1]] = 2*erg.hyp$value + 2*log(length(dataset))
                    names(erg.hyp)[length(erg.hyp)]="bic"
                    
                    return(erg.hyp)
                    
                  }
                  
                  #summe der k-1 gewichte kleiner gleich 1
                  m1 = c(rep(-1,k-1),rep(0,2*k))
                  c1 = -1
                  
                  #positivittsbedingugen gewichte
                  m2 = cbind(diag(k-1),matrix(0,ncol=2*k,nrow=k-1))
                  c2 = c(rep(0,k-1))
                  
                  #positivittsbedingugen sigma
                  m3 = cbind(matrix(0,ncol=2*k-1,nrow=k),diag(k))
                  c3 = c(rep(0,k))
                  
                  Mat = rbind(m1,m2,m3)
                  con = c(c1,c2,c3)
                  
                  
                  erg.hyp <- constrOptim(f=minus_log_like,theta=param,ui=Mat,ci=con,grad=NULL,k=k,x=dataset,cp=cp,csig=csig)#,control=list(trace=2))
                  
                  sigma.fit = erg.hyp$par[(2*k):(3*k-1)]
                  penaltyValue_sig = penalty_sig(sig=sigma.fit,csig=csig,x=dataset)
                  erg.hyp[[length(erg.hyp)+1]] = penaltyValue_sig
                  names(erg.hyp)[length(erg.hyp)]="penaltyValue_sig"
                  
                  p.fit = erg.hyp$par[1:(k-1)]
                  penaltyValue_p = penalty_p(p=p.fit,cp=cp)
                  erg.hyp[[length(erg.hyp)+1]] = penaltyValue_p
                  names(erg.hyp)[length(erg.hyp)]="penaltyValue_p"
                  
                  aic = 2*erg.hyp$value + 2*length(param)                    + 2*penaltyValue_sig + 2*penaltyValue_p
                  bic = 2*erg.hyp$value + length(param)*log(length(dataset)) + 2*penaltyValue_sig + 2*penaltyValue_p
                  
                  erg.hyp[[length(erg.hyp)+1]] = aic
                  names(erg.hyp)[length(erg.hyp)]="aic"
                  
                  erg.hyp[[length(erg.hyp)+1]] = bic
                  names(erg.hyp)[length(erg.hyp)]="bic"
                  
                  return(erg.hyp)
                }
                
                #mllk
                minus_log_like = function(x,param,k,cp,csig){
                  p   = param[1:(k-1)]
                  p[k]= 1-sum(p)
                  mu  = param[k:(2*k-1)]
                  sig = param[(2*k):(3*k-1)]
                  dens = numeric(length(x))
                  for(i in 1:k){
                    dens = dens + p[i]*dnorm(x,mean=mu[i],sd=sig[i])
                  }
                  pen_p = penalty_p(p[1:(k-1)],cp)
                  pen_sig = penalty_sig(sig,csig,x=x)
                  return(-(sum(log(dens)) + pen_sig + pen_p ))
                }
                
                #penalty function for weights
                penalty_p = function(p,cp){
                  p[(length(p)+1)] = 1 - sum(p)
                  return(cp*sum(log(p)))
                }
                
                #penalty function for std's
                penalty_sig = function(sig,csig,x){
                  n=length(x)
                  s_n_square = (n-1)/n * var(x)
                  erg=0
                  for(i in 1:length(sig)) erg[i]=(s_n_square/sig[i]^2 + log(sig[i]^2/s_n_square))
                  return(-csig*sum(erg))
                }
                
                #do mle estimation
                ergFit = mle_fitting(dataset=x,param=startParam,k=k,cp=penaltyP,csig=penaltySigma) 
                
                pTMP=ergFit$par[1:(k-1)]
                sigmaTMP=ergFit$par[(2*k):(3*k-1)]
                
                return(list(estimatedModel=createNormalMixtureModel(p=ergFit$par[1:(k-1)],
                                                                    mu=ergFit$par[k:(2*k-1)],sigma=ergFit$par[(2*k):(3*k-1)]),
                            loglik=c(-( ergFit$value +penalty_sig(sigmaTMP,penSig,x) + penalty_p(pTMP,penP) ),-ergFit$value),
                            AIC=ergFit$aic,BIC=ergFit$bic))
                
              }
              
              if(!missing(object)){
                mu=as.vector(object@mu)
                sigma=as.vector(object@sigma)                
                p=object@p[1:(k-1)]
              }
              else{mu=NULL; sigma=NULL; p=NULL}
            return(fitPenMLEONE(x=x,k=k,p=p,mu=mu,sigma=sigma,penaltyP=penP,penaltySigma=penSig))  
            }

            #multi-dimensional fitting
            if(length(x[1,]) > 1){
              
              #converting a symmetric or lower diagonal matrix to D*(D+1)/2 Vector
              mat2vec <- function(mat)
              {
                len <- ncol(mat)
                erg <- numeric(0.5*len*(len+1)) 
                for(i in 1:len)
                {
                  erg[(0.5*i*(i-1)+1):(0.5*i*(i+1))] <- mat[i,1:i]
                }
                return(erg)
              }
              
            
              convert_object <- function(object)
              {  
                mu_vek <- object@mu
                sigma_vek <- object@sigma
                p_vek <- object@p
                K <- length(mu_vek[1,])
                Dim <- length(mu_vek[,1])
                
                if(Dim == 1){stop("Only multivariate mixtures can be converted.");}
                
                anz_par <- K*Dim + K*choose(Dim+1,2) + K-1 # Anzahl der Parameter
                startpunkt <- numeric(anz_par)
                
                for(i in 1:K)
                {
                  startpunkt[(1 + Dim*(i-1)) : (Dim*i)] <- mu_vek[,i]
                  startpunkt[(K*Dim + 1 + (i-1)*Dim*(Dim+1)*0.5):(K*Dim + i*Dim*(Dim+1)*0.5)] <- mat2vec(t(chol(solve(sigma_vek[,,i]))))
                } 
                
                P = array(dim = c(K-1, K-1))
                for(i in 1:(K-1)){P[i,] <- p_vek[i];P[i,i] <- p_vek[i] - 1;}
                q_start <- try(sqrt(solve(P, -p_vek[1:(K-1)]))); 
                if(class(q_start) == "try-error") return("keinpunkt")
                startpunkt[(K*(Dim + 0.5*(Dim+1)*Dim) + 1):(K*(Dim + 0.5*(Dim+1)*Dim) + K - 1)] <- q_start;  
                return(startpunkt)
              }              
              
              if(!missing(object)){startpoint <- convert_object(object)}
              else{startpoint = NULL}
              
              
              newton_method <- function(x, 
                                        K = 2, 
                                        startpoint = NULL, 
                                        start_em = -1, #Voriteration mit EM (Anzahl, -1 heisst bis Konvergenz)
                                        start_grad = 20, #Voriteration mit Gradientenverf. falls start_em = -1 und prop = 1
                                        armijo_par = bcl, 
                                        prop = 0,
                                        pen = penSig,
                                        maxit = maxit, 
                                        parallel = parallel,
                                        verbose = verbose,
                                        doFI = doFI, #Ob Fisher Information berechnen
                                        hard_conv = hard_conv, #Ob alle drei Abrruchkriterien (delta, grad und rlc) gleichzeitig erfllt sein mssen (AND statt OR)
                                        return_vector = FALSE, #Ob der Rckgabewert ein Vektor der Elemente sein muss
                                        tol_eps = tol_eps, #Toleranz fuer LGS-Solver
                                        tol_delta = tol_delta, #Toleranz fuer Norm d. Schrittrichtung
                                        tol_grad = tol_grad, #Toleranz fuer Norm d. Gradienten
                                        tol_rlc = tol_rlc, #Toleranz fue rlc
                                        tol_em = tol_em #Toleranz fuer den EM-Algorithmus
              ){
                CholOrSOR <- 0 
                #####################################################
                #################### Hilfsfunktionen ################
                #####################################################
                
                #converting a D*(D+1)/2 Vector into a lowerdiagonal or symmetric matrix rowweise
                vec2mat <- function(vect, symm = FALSE)
                {
                  if(!symm)
                  {
                    len <- length(vect)
                    di = (-1 + sqrt(1+8*len))/2
                    erg <- matrix(0, ncol=di, nrow =di)
                    for(i in 1:di){for(j in 1:i){erg[i,j] <- vect[i*(i-1)/2 + j]}}
                  }
                  else
                  {
                    len <- length(vect)
                    di = (-1 + sqrt(1+8*len))/2
                    erg <- matrix(0, ncol=di, nrow =di)
                    for(i in 1:di){for(j in 1:i){erg[i,j] <- vect[i*(i-1)/2 + j]}}
                    erg_c <- t(erg)
                    diag(erg_c) <- 0
                    erg <- erg + erg_c
                  }
                  return(erg)
                }
                
                #converting a symmetric or lower diagonal matrix to D*(D+1)/2 Vector
                mat2vec <- function(mat)
                {
                  len <- ncol(mat)
                  erg <- numeric(0.5*len*(len+1)) 
                  for(i in 1:len)
                  {
                    erg[(0.5*i*(i-1)+1):(0.5*i*(i+1))] <- mat[i,1:i]
                  }
                  return(erg)
                }
                
                
                zs_delta <- function(i)
                {
                  z <- 0.5 + sqrt(0.25 + 2*i)
                  fl_z <- floor(z)
                  
                  if(fl_z == z) z <- z - 1
                  else z <- fl_z
                  
                  sp <- i- 0.5*z*(z-1)
                  return(c(z,sp))
                }
                
                #Zeile/Spalte von vec(A)_t bei Vektorisierung einer allgemeinen rechteckigen Matrix A
                zs_norm <- function(t,n)
                {
                  z <- ceiling(t/n)
                  s <- t - n*(z-1)
                  return(c(z,s))
                }
                
                c2r <- function(c_result, K, Dim, prop = 0)
                {
                  
                  if(!prop)   # falls kein proprtionaler Fall
                  {
                    grad_len <- K*(Dim + 0.5*Dim*(Dim+1)) + K - 1 
                    hess_len <- 0.5*grad_len*(grad_len+1)
                  }
                  
                  else
                  {
                    grad_len <- K*Dim + 0.5*(Dim+1)*Dim + 2*K - 2
                    hess_len <- 0.5*grad_len*(grad_len+1)  
                  }
                  
                  gradient <- c_result[1:(grad_len)]
                  hessian <- vec2mat(c_result[(grad_len+1):(grad_len + hess_len)])
                  
                  hessian <- hessian + t(hessian)*t(lower.tri(hessian))
                  
                  fval <- c_result[grad_len + hess_len + 1]
                  return(list("loglikelihood" = fval, "gradient" = gradient, "hessian" = hessian))
                }
                
                #Formel (17)
                k_delta  <- function(i,j){return(0.5*i*(i-1)+j)}
                
                # C-Version
                logli_derivatives <- function(parameter, x, prop = 0, pen = 0, grad = TRUE, hess = TRUE, parallel = parallel)
                {
                  Dim <- ncol(x)
                  N <- nrow(x)
                  
                  len <- length(parameter)
                  if(!prop) K <- (2*len + 2)/(3*Dim + Dim^2 + 2) # Komponentenanzahl
                  else K <- (len + 2 - 0.5*(Dim+1)*Dim)/(Dim+2)
                  
                  #len_L <- 0.5*Dim*(Dim+1)
                  output_len <- len*(len+1)*0.5 + len + 1;
                  the_output <- numeric(output_len)
                  
                  if(!pen) cova <- 0  #Stichprobenkovarianz nicht bentigt
                  else cova <- mat2vec(cov(x))
                  
                  if(!parallel)
                  {
                    erg_c <- .C("logli_derivatives",
                                parameter = as.double(parameter),
                                x = as.double(c(t(x))),
                                nrowp = as.integer(N),
                                Dimp = as.integer(Dim),
                                Kp = as.integer(K),
                                gradp = as.integer(grad),
                                hessp = as.integer(hess),
                                prop = as.integer(prop),
                                penalizep = as.double(pen),
                                cov = as.double(cova),
                                the_output  = as.double(the_output))
                  }
                  
                  else
                  {
                    erg_c <- .C("logli_derivatives_par",
                                parameter = as.double(parameter),
                                x = as.double(c(t(x))),
                                nrowp = as.integer(N),
                                Dimp = as.integer(Dim),
                                Kp = as.integer(K),
                                gradp = as.integer(grad),
                                hessp = as.integer(hess),
                                prop = as.integer(prop),
                                penalizep = as.double(pen),
                                cov = as.double(cova),
                                the_output  = as.double(the_output))
                  }
                  
                  #erg <- c2r(erg_c$the_output, K, Dim) 
                  #return(erg)
                  rm(the_output) 
                  return(erg_c$the_output) 
                }
                

                newton_direction <- function(parameter, x, prop = 0, pen = 0, cova = 0, CholOrSOR = 0, tol_eps = 1e-6, richtung, parallel = parallel)
                {
                  Dim <- ncol(x)
                  N <- nrow(x)
                  xvec <- c(t(x))
                  len <- length(parameter)
                  output_len = len*(len+1)*0.5 + len + 1
                  the_output = numeric(output_len)
                  
                  if(!prop) K <- (2*len + 2)/(3*Dim + Dim^2 + 2) # Komponentenanzahl
                  else K <- (len + 2 - 0.5*(Dim+1)*Dim)/(Dim+2)
                  
                  if(pen != 0 && cova[1] == 0)
                  {
                    cat("\nBitte Uebergeben Sie die Kovarianzmatrix (als Vektor) in newton_direction().\n")
                    cova <- mat2vec(cov(x))
                  }
                  
                  
                  if(!parallel)
                  {
                    erg <- .C("newton_direction",
                              parameter = as.double(parameter),
                              x = as.double(xvec),
                              nrowp = as.integer(N),
                              Dimp = as.integer(Dim),
                              Kp = as.integer(K),
                              prop = as.integer(prop),
                              penalize = as.double(pen),
                              cov = as.double(cova),
                              the_output  = as.double(the_output),
                              tol_eps = as.double(tol_eps),
                              CholOrSOR = as.integer(CholOrSOR),
                              richtung = as.double(richtung)
                    )
                  }
                  
                  else
                  {
                    erg <- .C("newton_direction_par",
                              parameter = as.double(parameter),
                              x = as.double(xvec),
                              nrowp = as.integer(N),
                              Dimp = as.integer(Dim),
                              Kp = as.integer(K),
                              prop = as.integer(prop),
                              penalize = as.double(pen),
                              cov = as.double(cova),
                              the_output  = as.double(the_output),
                              tol_eps = as.double(tol_eps),
                              CholOrSOR = as.integer(CholOrSOR),
                              richtung = as.double(richtung)
                    )
                  }
                  
                  return(erg)
                }
                
                solve_c <- function(devs, anz_par, tol_eps)
                {
                  richtung <- numeric(anz_par)
                  erg  <- .C("solve", as.double(devs), as.integer(1), as.integer(anz_par), as.double(tol_eps), as.double(richtung))
                  return(erg)
                }
                
                # Siehe Seiten 89,90 W. Alt
                armijo <- function(x, sig, g, delta, anz_max, startpunkt, richtung, loglike_val, gr, prop = 0, pen = 0)
                {
                  anz_it <- 0
                  ex <- 0
                  len <- length(startpunkt)
                  like_ind <- len + len*(len+1)*0.5 + 1
                  
                  while(anz_it <= anz_max)
                  {
                    #cat("Armijo-Iteration.")
                    anz_it <- anz_it + 1
                    schritt <- sig*(g^ex)
                    ex <- ex + 1
                    startpunkt_neu <- startpunkt + schritt*richtung
                    loglike_val <- logli_derivatives(startpunkt_neu, x, prop = prop, pen = pen, grad = FALSE, hess = FALSE, parallel = parallel)[like_ind]
                    
                    if(loglike_val >= loglike_val)
                    {
                      return(schritt)
                    }
                  }
                  return(-1)
                }
                
                #Nocedal, Wright, S. 37
                backtracking <- function(x, richtung, startpunkt, gr, loglike_val, rho = 0.7, c = 1e-4, schritt = 1, anz_max = 35,  prop = 0, pen = 0)
                {
                  # x - Daten, richtung klar
                  # gr -Skalarprodukt richtung*gradient
                  # startpunkt - aktuelle Schtung (x_k)
                  # loglike_val - Wert von f an x_k
                  
                  anz_it <- 0
                  len <- length(startpunkt)
                  like_ind <- len + len*(len+1)*0.5 + 1
                  
                  while(anz_it <= anz_max)
                  {
                    #cat("Armijo-Iteration.")
                    anz_it <- anz_it + 1
                    schritt <- schritt*rho
                    startpunkt_neu <- startpunkt + schritt*richtung
                    loglike_neu <- logli_derivatives(startpunkt_neu, x, prop = prop, pen = pen, grad = FALSE, hess = FALSE, parallel = parallel)[like_ind]
                    
                    if(loglike_neu >= loglike_val + c*schritt*gr)
                    {
                      return(schritt)
                    }
                  }
                  
                  return(-1)
                }
                
                # Erstellt Rückgabeobjekt
                return_erg <- function(startpunkt, prop = 0, Dim, K, logli, q_vec, anz_par, iter, anz_dat, conv = "convergence", doFI = FALSE, x = 0, parallel = parallel, return_vector = FALSE)
                {
                  if(return_vector){return(startpunkt);}
                  
                  mu_vec <- array(dim = c(Dim,K), data = 0);
                  p_vec <- numeric(K) # Vektor f�r die Gewichte p
                  nenner <- sum(startpunkt[q_vec]^2) + 1 # Nenner der Gewichte
                  nenner2 <- nenner^2
                  p_vec[1:(K-1)] <- startpunkt[q_vec]^2 / nenner
                  p_vec[K] <- 1 - sum(p_vec[1:(K-1)]) # letztes Gewicht
                  
                  if(!prop)
                  {
                    sigma_vec <- array(dim = c(Dim,Dim,K), data = 0);
                    
                    for(l in 1:K)
                    {
                      mu_vec[,l] <- startpunkt[(1 + Dim*(l-1)) : (Dim*l)]
                      sigma_vec[,,l] <- vec2mat(startpunkt[(K*Dim + 1 + (l-1)*Dim*(Dim+1)*0.5):(K*Dim + l*Dim*(Dim+1)*0.5)])
                      sigma_vec[,,l] <- solve(sigma_vec[,,l] %*% t(sigma_vec[,,l]))
                    }
                    
                    bic <- -2*logli[1] + anz_par*log(anz_dat)
                    
                    FI <- "Covariance not calculated."
                    
                    if(doFI)
                    {
                      ld <- logli_derivatives(startpunkt, x, prop = prop, pen = 0, grad = TRUE, hess = TRUE, parallel =  parallel)
                      FI <- fisherInfo(startpunkt, ld, Dim, K, anz_dat)
                    }
                    
                    objectEst = createNormalMixtureModel(p=p_vec[1:(K-1)],mu=mu_vec,sigma=sigma_vec)
                    
                    return(list(estimatedModel=objectEst, "BIC" = bic, "loglik" = logli, "numit" = iter, "convergence" = conv, "MLE_covariance" = FI))
                  }
                  
                  
                  else #proportional case
                  {
                    for(l in 1:K)
                    {
                      mu_vec[,l] <- startpunkt[(1 + Dim*(l-1)) : (Dim*l)]
                    }
                    
                    sigma <- vec2mat(startpunkt[(K*Dim + 1):(K*Dim + Dim*(Dim+1)*0.5)])
                    sigma <- solve(sigma %*% t(sigma))
                    lambda <- startpunkt[(K*Dim + 0.5*(Dim+1)*Dim + 1):(K*Dim + 0.5*(Dim+1)*Dim + K - 1)]
                    bic <- -2*logli + anz_par*log(anz_dat)
                    
                    FI <- "Covariance not calculated."
                    
                    if(doFI)
                    {
                      ld <- logli_derivatives(startpunkt, x, prop = prop, pen = 0, grad = TRUE, hess = TRUE, parallel =  parallel)
                      FI <- fisherInfo(startpunkt, ld, Dim, K, prop = TRUE, anz_dat)
                    }
                    return(list("mean" = mu_vec, "variance" = sigma, "prop" = 1/lambda^2,  "weight" = p_vec, "BIC" = bic, "loglik" = logli, "numit" = iter, "convergence" = conv, "MLE covariance" = FI))
                  }
                }
                
                
                # Fr Deltamethode
                # L untere Dreiecksmatrix
                # A = (LL^T)^-1
                for_delta_L <- function(L, A, zs_mat_delta, zs_mat_norm, i, j)
                {
                  #L <- vec2mat(L)
                  n <- nrow(L)
                  #A <- L %*% t(L)
                  #A <- solve(A)
                  
                  #iz <- zs_delta(i)
                  iz <- zs_mat_delta[i,]
                  is <- iz[2]
                  iz <- iz[1]
                  
                  #jz <- zs_delta(j)
                  jz <- zs_mat_delta[j,] 
                  js <- jz[2]
                  jz <- jz[1]  
                  erg <- 0
                  
                  for(t in 1:n^2)
                  {
                    tz <- zs_mat_norm[t,] 
                    ts <- tz[2] 
                    tz <- tz[1]
                    
                    dev_inv <- -A[iz, tz]*A[ts,is]
                    temp <- (tz==jz)*L[ts,js] + (ts==jz)*L[tz,js]
                    
                    #if((ts < js) || tz < js)
                    #cat("\n ups \n")
                    #temp <- temp + (tz==jz)*(ts < js)*L[js,ts] + (ts==jz)*(tz < js)*L[js,tz]
                    erg <- erg + dev_inv*temp
                  }
                  return(erg)
                }
                
                
                ## Functions for the DeltaMethod
                
                #Transformationsmatrix for the original Fisherinformation
                transformation_matrix <- function(parameter, Dim, K, prop = FALSE)
                {
                  num_els <- Dim*(Dim+1)*0.5 # Anzahl der Elemente in L
                  zs_mat_delta <- array(dim = c(num_els,2))
                  zs_mat_norm <- array(dim = c(Dim^2,2))
                  anz_par <- length(parameter)
                  
                  for(i in 1:num_els)
                  {
                    zs_mat_delta[i,] <- zs_delta(i) 
                  }
                  
                  for(i in 1:(Dim^2))
                  {
                    zs_mat_norm[i,] <- zs_norm(i,Dim)
                  }
                  
                  
                  trafo_mat <- array(dim = c(anz_par,anz_par), data = 0) 
                  trafo_mat[1:(K*Dim),1:(K*Dim)] <- diag(K*Dim)
                  
                  if(!prop) 
                  {
                    #Ableitungen nach L 
                    for(i in 1:K)
                    {
                      Lu <- (K*Dim + (i-1)*num_els + 1)
                      Lo <- (K*Dim + i*num_els)   
                      L <- vec2mat(parameter[Lu:Lo])
                      A <- solve(L %*% t(L))
                      
                      #Ableitungen von (LL^T)^-1 reinschreiben
                      for(k in 1:num_els)
                      {
                        for(l in 1:num_els)
                        {
                          trafo_mat[(Lu + k - 1), (Lu + l - 1)] <- for_delta_L(L, A, zs_mat_delta, zs_mat_norm, k, l)
                        }
                      }          
                    }
                    
                    pu <- K*(Dim + 0.5*Dim*(Dim+1)) + 1
                    qvec <- abs(parameter[pu:(pu+K-2)])
                    qvec2 <- qvec^2
                    sumqvec2 <- sum(qvec2)
                    
                    for(i in 1:(K-1))
                    {
                      
                      vorfaktor <- -2*qvec[i]/(sumqvec2+1)^2
                      qvec2i <- qvec2
                      qvec2i[i] <- -sumqvec2 + qvec2[i] - 1
                      #Ableitungen nach von p
                      trafo_mat[pu + i - 1, pu:(pu+K-2)] <-  vorfaktor*qvec2i
                    }
                  }
                  
                  else # Falls proportionaler Fall
                  {
                    Lu <- K*Dim + 1
                    Lo <- K*Dim + num_els  
                    L <- vec2mat(parameter[Lu:Lo])
                    A <- solve(L %*% t(L))
                    
                    for(k in 1:num_els)
                    {
                      for(l in 1:num_els)
                      {
                        trafo_mat[(Lu + k - 1), (Lu + l - 1)] <- for_delta_L(L, A, zs_mat_delta, zs_mat_norm, k, l)
                      }
                    }  
                    lambdau <- K*Dim + 0.5*Dim*(Dim+1) + 1
                    pu <- lambdau + K - 1
                    qvec <- abs(parameter[pu:(pu+K-2)])
                    qvec2 <- qvec^2
                    sumqvec2 <- sum(qvec2)
                    
                    for(i in 1:(K-1))
                    {
                      
                      trafo_mat[lambdau + i - 1, lambdau + i - 1] <- -0.5*(parameter[lambdau + i - 1])^(-3/2)
                      vorfaktor <- -2*qvec[i]/(sumqvec2+1)^2
                      qvec2i <- qvec2
                      qvec2i[i] <- -sumqvec2 + qvec2[i] - 1
                      #Ableitungen nach von p
                      trafo_mat[pu + i - 1, pu:(pu+K-2)] <-  vorfaktor*qvec2i
                    }
                  }
                  
                  return(trafo_mat)    
                }
                
                fisherInfo <- function(parameter, ld, Dim, K, samplesize, prop = FALSE)
                {
                  if(!prop)  hu <- K*(Dim + 0.5*Dim*(Dim+1)) + K - 1  #Index wo die Hessematrix im Ergebnis von logli_derivatives anf?ngt
                  else  hu <- K*Dim + 0.5*Dim*(Dim+1) + 2*K - 2  
                  ho <- hu*(hu+1)*0.5 #L?nge der Hessematrix als Vektor    
                  ll_hessian <- vec2mat(ld[(hu+1):(hu+ho)], symm = TRUE)
                  trafo <- transformation_matrix(parameter, Dim, K, prop = prop)  
                  tr_len <- length(trafo[1,])
                  trafo <- try(solve(trafo)) 
                  if(class(trafo) == "try-error"){warning("Transformation matrix could not be calculated. System singular. Returning Fisher-Information for untransformed parameters."); trafo <- diag(tr_len);}  
                  fisherInfo_erg <- try( -1 * t(trafo) %*% ll_hessian %*% trafo / samplesize)
                  return(fisherInfo_erg)
                }
                
                
                
                mclust_parameter <- function(startpunkt, Dim, K)
                {
                  modelName <- "VVV"
                  q_vec <- K*(Dim + 0.5*(Dim+1)*Dim) + 1:(K-1)
                  p_vec <- numeric(K)
                  nenner <- sum(startpunkt[q_vec]^2) + 1 # Nenner der Gewichte
                  p_vec[1:(K-1)] <- startpunkt[q_vec]^2 / nenner
                  p_vec[K] <- 1 - sum(p_vec[1:(K-1)]) # letztes Gewicht
                  
                  means <- array(dim = c(Dim,K))
                  sigma_vec <- array(dim = c(Dim,Dim,K))
                  chol_vec <- array(dim = c(Dim,Dim,K))
                  
                  
                  for(i in 1:K)
                  {
                    means[,i] <- startpunkt[((i-1)*Dim + 1) : (i*Dim)]
                    sigma_vec[,,i] <- vec2mat(startpunkt[(K*Dim + 1 + (i-1)*Dim*(Dim+1)*0.5):(K*Dim + i*Dim*(Dim+1)*0.5)])
                    sigma_vec[,,i] <- solve(sigma_vec[,,i] %*% t(sigma_vec[,,i]))
                    chol_vec[,,i] <- chol(sigma_vec[,,i])
                  }
                  
                  variance <- list("modelName" = "VVV", "d" = Dim, "G" = K, "sigma" = sigma_vec, "cholsigma" = chol_vec)
                  return(list("pro" = p_vec, "mean" = means,  "variance" = variance))
                }
                
                
                mclust_parameter_inverse <- function(em_object, Dim, K)
                {
                  parameter <- numeric(K*(Dim + 0.5*Dim*(Dim + 1)) + K - 1);
                  P = array(dim = c(K-1, K-1))
                  
                  for(i in 1:(K-1)){P[i,] <- em_object$parameters$pro[i]; P[i,i] <- em_object$parameters$pro[i] - 1;}
                  parameter[(K*(Dim + 0.5*Dim*(Dim + 1)) + 1) : (K*(Dim + 0.5*Dim*(Dim + 1)) + K - 1)] <- sqrt(solve(P, -em_object$parameters$pro[1:(K-1)]));
                  
                  for(i in 1:K)
                  {
                    parameter[((i-1)*Dim + 1):(i*Dim)] <- em_object$parameters$mean[,i]
                    
                    parameter[(K*Dim + (i-1)*Dim*(Dim+1)*0.5+1):(K*Dim + i*Dim*(Dim+1)*0.5)] <- mat2vec(t(chol(solve(em_object$parameters$variance$sigma[,,i]))));
                  }
                  
                  return(parameter)
                }
                
                
                em_iterations <- function(x, startpunkt, itmax, Dim, K, tol = c(1.e-8, sqrt(.Machine$double.eps)))
                {  
                  start_par <- mclust_parameter(startpunkt, Dim, K)
                  erg_em <- em(modelName = "VVV", data = x, parameters = start_par, control = emControl(itmax = itmax, tol = tol))
                  parameter <- mclust_parameter_inverse(erg_em, Dim, K)  
                  info <-  attr(erg_em, "info")
                  return(list("parameter" = parameter, "numit" = info[1]));
                }
                
                
                ###########################################
                ########## Ende Hilfsfunktionen ###########
                ###########################################
                
                if(K < 2) stop("The number must be at least 2.")
                if(doFI) xFi <- x
                else xFi <- 0
                if(Sys.info()["sysname"] == "Windows") parallel = FALSE # unter windows in jedem Fall nicht parallel
                
                Dim <- ncol(x)
                anz <- nrow(x)
                
                if(pen == "n"){pen = 1/anz} #Penalisierungsterm
                if(pen == "sn"){pen = 1/sqrt(anz)}
                if(pen != 0){cova <- mat2vec(cov(x))}
                else{cova <- 0}
                
                if(!prop)
                {
                  anz_par <- K*Dim + K*choose(Dim+1,2) + K-1 # Anzahl der Parameter
                  L = array(dim = c(Dim,Dim,K))
                  sigma_vec <- array(dim = c(Dim, Dim, K), data = 0);
                  q_vec <- K*(Dim + 0.5*(Dim+1)*Dim) + 1:(K-1)
                }
                
                else
                {
                  anz_par <- K*Dim + choose(Dim+1,2) + 2*K - 2
                  L = array(dim = c(Dim,Dim,1))
                  sigma_vec <- array(dim = c(Dim, Dim, 1), data = 0);
                  q_vec <- K*Dim + 0.5*(Dim+1)*Dim + K - 1 + 1:(K-1)
                }
                
                startpunkt <- numeric(anz_par)
                richtung <- numeric(anz_par)
                like_ind <- anz_par + 0.5*anz_par*(anz_par+1) + 1 # Index des Funktionswertes in erg_c 
                p_vec <- numeric(K) # Vektor fuer die Gewichte p
                
                if(is.null(startpoint))  #Falls kein Startpunkt �bergeben wurde - kmeans
                {
                  km <- kmeans(x, centers = K)
                  
                  if(!prop)
                  {
                    # Cholesky-Faktoren der Inversen der Kovarianzen definieren
                    for(i in 1:K){L[,,i] = t(chol(solve(cov(x[km$cluster == i,]))))}
                    
                    for(i in 1:K)
                    {
                      startpunkt[(1 + Dim*(i-1)) : (Dim*i)] <- km$centers[i,]
                      startpunkt[(K*Dim + 1 + (i-1)*Dim*(Dim+1)*0.5):(K*Dim + i*Dim*(Dim+1)*0.5)] <- mat2vec(L[,,i])
                      #if(i < K )startpunkt[K*(Dim + 0.5*(Dim+1)*Dim) + i] <- km$size[1:(K-1)]/anz
                    }
                    
                    
                    ## Jetzt Startwerte fuer q berechnen:
                    p_vec <- km$size[1:(K-1)]/anz
                    P = array(dim = c(K-1, K-1))
                    for(i in 1:(K-1)){P[i,] <- p_vec[i];P[i,i] <- p_vec[i] - 1;}
                    q_start <- sqrt(solve(P, -p_vec[1:(K-1)]));
                    startpunkt[(K*(Dim + 0.5*(Dim+1)*Dim) + 1):(K*(Dim + 0.5*(Dim+1)*Dim) + K - 1)] <- q_start;
                  }
                  
                  else
                  {
                    # Cholesky-Faktoren der Inversen der Kovarianzen definieren
                    L[,,1] = t(chol(solve(cov(x[km$cluster == 1,]))))
                    
                    # Proportionalitaetsfaktoren
                    for(i in 1:(K-1))
                    {
                      temp_mat <- t(chol(solve(cov(x[km$cluster == i+1,]))))
                      startpunkt[K*Dim + 0.5*Dim*(Dim+1) + i] <- sqrt(temp_mat[1,1]/L[1,1,1])
                    }
                    
                    # Mittelwerte
                    for(i in 1:K)
                    {
                      startpunkt[(1 + Dim*(i-1)) : (Dim*i)] <- km$centers[i,]
                    }
                    
                    # Kovarianz
                    startpunkt[(K*Dim + 1):(K*Dim + Dim*(Dim+1)*0.5)] <- mat2vec(L[,,1])
                    
                    # Gewichte
                    p_vec <- km$size[1:(K-1)]/anz
                    P = array(dim = c(K-1, K-1))
                    for(i in 1:(K-1)){P[i,] <- p_vec[i];P[i,i] <- p_vec[i] - 1;}
                    q_start <- sqrt(solve(P, -p_vec[1:(K-1)]));
                    startpunkt[(K*Dim + 0.5*(Dim+1)*Dim + K):(K*Dim + 0.5*(Dim+1)*Dim + 2*K - 2)] <- q_start;
                  }
                  
                } #Ende kmeans
                
                else startpunkt <- startpoint
                
                
                # Iterationen
                
                #starpunkt <- startpunkt + runif(length(startpunkt), 1, 2)
                #Hier mit Gradienten-Verfahren voriterieren
                contraction <- FALSE # Ob Kontraktionsgebiet erreicht wurde 
                loglike_val <- -1e200
                
                if(start_em > 0 || (start_em == -1 && prop)) 
                {  
                  numitEM <- 0
                  if(!prop)
                  {
                    r_norm_0 <- -Inf
                    itz <- 0
                    #while(!contraction) # Einen anderen Ansatz berlgegen wann man mit Newton bernehmen darf
{  
  startpunkt <- suppressWarnings(em_iterations(x, startpunkt, start_em, Dim, K))
  numitEM <- numitEM + startpunkt$numit          
  startpunkt <- startpunkt$parameter
  richtung <- numeric(anz_par)
  nr <- newton_direction(startpunkt, x, prop, pen = pen, cova = cova, CholOrSOR, tol_eps = tol_eps, richtung, parallel = parallel) #tol_eps 0 bei LGS solve
  richtung <- nr[[12]]
  grad  <- nr[[9]]; grad <- grad[1:anz_par]      
  gr <- richtung %*% grad
  
  if(verbose)cat(paste("Ableitung in Newton-Richtung: ", gr, "\n", sep =""))
  
  loglike_val <- (nr[[9]])[like_ind];
} # Ende while(!contraction)
                    
                  }
                  
                  # Im proportionallen Fall Gradientenverfahren mit Armijo
                  else 
                  {  
                    if(start_em == -1) start_em <- start_grad
                    numitEM <- start_grad
                    
                    for(k in 1:start_em)
                    {
                      devs <- logli_derivatives(startpunkt, x, hess = FALSE, prop = 1, parallel = parallel)
                      grad <- devs[1:anz_par]
                      loglike_val <- devs[like_ind]
                      gradnorm <- norm(matrix(grad),"F")
                      
                      
                      if(gradnorm > tol_grad)
                      {
                        gr <- grad %*% grad/gradnorm
                        grad <- grad/gradnorm  
                        #schritt <- armijo(x, sig, g, delta, anz_max, startpunkt, grad, loglike_val, gradnorm, prop = prop, pen = pen)
                        schritt <- backtracking(x, grad, startpunkt, gr = gr, loglike_val = loglike_val, rho = 0.8, c = 1e-4, schritt = 1.5, anz_max = armijo_par,  prop = 0, pen = 0)
                        if(schritt != -1) startpunkt <- startpunkt + schritt*grad
                      }
                      else  k <- start_em     
                    }
                  }
                  
                  ## Falls der Gradient schon verschwindet ##
                  if(max(abs(grad)) < tol_grad)
                  {
                    #cat("Norm of the gradient: \n")
                    #print(norm(matrix(grad)))
                    #cat("Norm of the direction: \n")
                    #print(0)
                    if(verbose)cat("Stopping rule: gradient ascent.\n")
                    
                    #Nichtpenalisierte Likelihood
                    if(pen != 0)
                    {
                      ld <- logli_derivatives(startpunkt, x, prop = prop, pen = 0, grad = TRUE, hess = TRUE, parallel = parallel)
                      ld <- ld[like_ind]
                      return(return_erg(startpunkt, prop = prop, Dim, K, c(ld,loglike_val), q_vec, anz_par, c(numitEM,0), anz, doFI = doFI, x = x, return_vector = return_vector))
                    }
                    
                    return(return_erg(startpunkt, prop = prop, Dim, K, loglike_val, q_vec, anz_par, c(numitEM,0), anz, doFI = doFI, x = x, return_vector = return_vector))
                  }
                } # Ende if start_em 
                
                if(start_em == -1 && !prop) #Erstmal komplett EM und dann Newton
                {
                  startpunkt <- suppressWarnings(em_iterations(x, startpunkt, Inf, Dim, K, tol = c(tol_em, sqrt(.Machine$double.eps))))
                  numitEM <- startpunkt$numit 
                  startpunkt <- startpunkt$parameter
                }  
                
                ###### HAUPTSCHLEIFE ######
                for(iter in 1:maxit)
                {     
                  nr <- newton_direction(startpunkt, x, prop, pen = pen, cova = cova, CholOrSOR, tol_eps = tol_eps, richtung, parallel = parallel) #tol_eps 0 bei LGS solve
                  richtung <- nr[[12]]
		  
 		
                  grad <- nr[[9]]; grad <- grad[1:anz_par]
                  loglike_val_o <- loglike_val #old value     
                  loglike_val <- (nr[[9]])[like_ind]
                  
                  if(richtung[1] == -12345678)
                  {
                    if(verbose)
                    {
                      cat("Stopping rule: hessian singular.\n")
                      cat(paste("Gradient norm: ", norm(matrix(grad)),"\n"))
                      cat(paste("Direction norm: ", norm(matrix(richtung)),"\n"))
                    }
                    
                    if(pen)
                    {
                      ld <- logli_derivatives(startpunkt, x, prop = prop, pen = 0, grad = FALSE, hess = FALSE, parallel = parallel)
                      ld <- ld[like_ind]
                      return(return_erg(startpunkt, prop = prop, Dim, K, c(ld,loglike_val), q_vec, anz_par, c(numitEM,0), anz, conv = "hessian singular", doFI = doFI, x = xFi, return_vector = return_vector))
                    }
                    
                    return(return_erg(startpunkt, prop = prop, Dim, K, loglike_val, q_vec, anz_par, c(numitEM,0), anz, conv = "hessian singular", doFI = doFI, x = xFi, return_vector = return_vector))
                  }
                  
                  if(!hard_conv) is_conv <- ((max(abs(richtung)) < tol_delta) | (max(abs(grad)) < tol_grad))
                  else is_conv <- (max(abs(richtung)) < tol_delta) & (max(abs(grad)) < tol_grad)
                  
                  rlc <- abs((loglike_val - loglike_val_o)/loglike_val_o) # relative change in loglikelihood
                  
                  #cat("\nRelative logli change: \n")
                  #print(rlc)
                  is_conv <- !hard_conv & (is_conv | (rlc < tol_rlc)) | hard_conv & (is_conv & (rlc < tol_rlc))
                  
                  #print(loglike_val_o)
                  
                  if(is_conv)
                  {
                    
                    #abs((loglike_val - loglike_val)/loglike_val) < tol_delta
                    if(verbose)
                    {
                      cat("Stopping rule: convergence.\n")
                      cat(paste("Gradient norm: ", norm(matrix(grad)),"\n"))
                      cat(paste("Direction norm: ", norm(matrix(richtung)),"\n"))
                      cat(paste("Rel. loglikelihood change: ", rlc,"\n"))          
                    }
                    
                    if(pen) #falls Penalisierung aktiviert, Likelihoodwert ohne Penalisierung berechnen
                      
                    {
                      ld <- logli_derivatives(startpunkt, x, prop = prop, pen = 0, grad = FALSE, hess = FALSE, parallel = parallel)
                      ld <- ld[like_ind]
                      return(return_erg(startpunkt, prop = prop, Dim, K, c(ld,loglike_val), q_vec, anz_par, c(numitEM, iter), anz, doFI = doFI, x = xFi, return_vector = return_vector))
                    }
                    
                    return(return_erg(startpunkt, prop = prop, Dim, K, loglike_val, q_vec, anz_par, c(numitEM, iter), anz, doFI = doFI, x = xFi, return_vector = return_vector)) 
                  }
                  
                  #loglike_val <- loglike_val
                  #cat(paste("Direction norm: ", norm(matrix(richtung)),"\n"))
                  
                  gr <- grad %*% richtung  
                  
                  if(gr < -tol_grad)   #falls Abstiegsrichtung Gradientenschritt
                  {
                    
                    #cat("Stopping rule: descent direction found.\n")
                    #cat(paste("Gradient norm: ", gradnorm,"\n"))
                    #cat(paste("Direction norm: ", norm(matrix(richtung)),"\n"))
                    #return(return_erg(startpunkt, prop = prop, Dim, K, loglike_val, q_vec, anz_par, iter, anz))
                    
                    gradnorm <- norm(matrix(grad),"F")
                    grad <- grad/gradnorm
                    schritt <- backtracking(x, grad, startpunkt, gr, loglike_val = loglike_val, rho = 0.7, c = 1e-4, schritt = 1, anz_max = armijo_par,  prop = 0, pen = 0)
                    #schritt <- armijo(x, sig, g, delta, anz_max, startpunkt, grad, loglike_val, gradnorm, prop = prop, pen = pen)
                    
                    
                    if(schritt == -1)
                    {
                      if(verbose)
                      {
                        cat("Stopping rule: no positive step size.\n")
                        cat(paste("Gradient norm: ", gradnorm,"\n"))
                        cat(paste("Direction norm: ", norm(matrix(richtung)),"\n")) 
                        cat(paste("Rel. loglikelihood change: ", rlc,"\n"))    
                      }
                      
                      
                      if(pen != 0)
                      {
                        ld <- logli_derivatives(startpunkt, x, prop = prop, pen = 0, grad = FALSE, hess = FALSE, parallel = parallel)
                        ld <- ld[like_ind]
                        return(return_erg(startpunkt, prop = prop, Dim, K, c(ld,loglike_val), q_vec, anz_par, c(numitEM, iter), anz, conv = "no positive step size", doFI = doFI, x = xFi, return_vector = return_vector))
                      }        
                      
                      return(return_erg(startpunkt, prop = prop, Dim, K, loglike_val, q_vec, anz_par, c(numitEM,iter), anz, conv = "no positive step size", doFI = doFI, x = xFi, return_vector = return_vector))
                    }
                    startpunkt <- startpunkt + schritt*grad
                    next; #nächste Iteration
                  }#Ende falls Abstiegsrichtung
                  
                  
                  
                  ### Armijo-Schrittweite mit Backtracking ###
                  schritt <- backtracking(x, richtung, startpunkt, gr, loglike_val = loglike_val, rho = 0.7, c = 1e-4, schritt = 1, anz_max = armijo_par,  prop = 0, pen = 0)
                  
                  if(schritt == -1)
                  {
                    if(verbose) 
                    {
                      cat("Stopping rule: no positive step size.\n")
                      cat(paste("Gradient norm: ", norm(matrix(grad)),"\n"))
                      cat(paste("Direction norm: ", norm(matrix(richtung)),"\n"))
                      cat(paste("Rel. loglikelihood change: ", rlc,"\n"))    
                    }
                    
                    if(pen != 0)
                    {
                      ld <- logli_derivatives(startpunkt, x, prop = prop, pen = 0, grad = FALSE, hess = FALSE, parallel = parallel)
                      ld <- ld[like_ind]
                      return(return_erg(startpunkt, prop = prop, Dim, K, c(ld,loglike_val), q_vec, anz_par, c(numitEM, iter), anz, conv = "no positive step size", doFI = doFI, x = xFi, return_vector = return_vector))
                    }
                    
                    return(return_erg(startpunkt, prop = prop, Dim, K, loglike_val, q_vec, anz_par, c(numitEM, iter), anz, conv = "no positive step size", doFI = doFI, x = xFi, return_vector = return_vector))
                  }
                  
                  startpunkt <- startpunkt + schritt*richtung
                }# Ende Hauptschleife
                
                if(verbose)
                {
                  cat("Stopping rule: maximal number of iterations achieved.\n")
                  cat(paste("Gradient norm: ", norm(matrix(grad)),"\n"))
                  cat(paste("Direction norm: ", norm(matrix(richtung)),"\n"))
                }
                
                if(pen != 0)
                {
                  ld <- logli_derivatives(startpunkt, x, prop = prop, pen = 0, grad = FALSE, hess = FALSE, parallel = parallel)
                  ld <- ld[like_ind]
                  return(return_erg(startpunkt, prop = prop, Dim, K, c(ld,loglike_val), q_vec, anz_par, c(numitEM, iter), anz, conv = "max. num. iterations", doFI =  doFI, x = xFi, return_vector = return_vector))
                }
                return(return_erg(startpunkt, prop = prop, Dim, K, loglike_val, q_vec, anz_par, c(numitEM, iter), anz, conv = "max. num. iterations", doFI = doFI, x = xFi, return_vector = return_vector))
                
        }#Ende newton_method
              
              
              
              return(newton_method(x,
                                   K = k, 
                                   startpoint = startpoint, 
                                   start_em = -1, #Voriteration mit EM (Anzahl, -1 heisst bis Konvergenz)
                                   start_grad = 200, #Voriteration mit Gradientenverf. falls start_em = -1 und prop = 1
                                   armijo_par = bcl, 
                                   prop = 0,
                                   pen = penSig,
                                   maxit = maxit, 
                                   parallel = parallel,
                                   verbose = verbose,
                                   doFI = doFI, #Ob Fisher Information berechnen
                                   hard_conv = hard_conv, #Ob alle drei Abrruchkriterien (delta, grad und rlc) gleichzeitig erfllt sein mssen (AND statt OR)
                                   return_vector = FALSE, #Ob der Rckgabewert ein Vektor der Elemente sein muss
                                   tol_eps = tol_eps, #Toleranz fuer LGS-Solver
                                   tol_delta = tol_delta, #Toleranz fuer Norm d. Schrittrichtung
                                   tol_grad = tol_grad, #Toleranz fuer Norm d. Gradienten
                                   tol_rlc = tol_rlc, #Toleranz fue rlc
                                   tol_em = tol_em #Toleranz fuer den EM-Algorithmus
              ))
              
            } 
            
     }
            


#define show method for mixture class
setMethod("show",signature(object="normalMixtureModel"),function(object){
  if(object@dimension>1){
    m=dim(object@mu)[2]
    cat("\n",object@dimension,"- dimensional normal mixture model with", m,"components.\n\n")     
  }
  else{
    m=length(object@mu)
    cat("\none-dimensional normal mixture model with", m,"components:\n")
    cat("\n \tp:\t\t\t\t\t",round(object@p,4))
    cat("\n \tmu:\t\t\t\t",round(object@mu,4))
    cat("\n \tsigma:\t",round(object@sigma,4),"\n\n")
  }
} )

#define create function for mixture class, only first m-1 values of p
createNormalMixtureModel=function(p,mu,sigma){
  if(is.vector(mu)) dimension=1
  else dimension=length(mu[,1])  
  
  if(dimension==1 && length(sigma)==1)
    stop("Please select a mixture with at least two components.")
  if(dimension>1 && is.vector(sigma[1,,]))
    stop("Please select a mixture with at least two components.")
  
  m=length(p)+1
  p=c(p,1-sum(p))
  #if(length(p)!=length(mu)) p = c(p,1-sum(p))
  
  #also vectors are possible as input in case of one dimension
  if(dimension==1){
    if(length(mu)!=length(sigma)||(length(p))!=length(sigma)||(length(p))!=length(mu))
      warning("dimension mismatch!")
    sigma=array(sigma,dim=c(1,1,m))
    mu=matrix(mu,ncol=m)

  }
  return(new("normalMixtureModel",p=p,mu=mu,sigma=sigma,dimension=dimension))
}


#define generic plot function
setMethod(f="plot",signature(x="normalMixtureModel"),
          definition=function(x,add,col,surface=TRUE){
            

            if(x@dimension==2 && surface==TRUE) {

              
              dnv <- function(x, mu, sigma, inverted = FALSE)
              {
                if(inverted) sigmainv = sigma
                else sigmainv = solve(sigma)
                mq <- mahalanobis(x,mu,sigmainv,TRUE)
                dime <- -length(x)
                con <- sqrt(2*pi)^dime
                dee <- sqrt(det(sigmainv))
                erg <- dee*con*exp(-0.5*mq)
                return(erg) 
              }

              
              #allgemeine Gauss-Mischung
              gaussmix <- function(x, mu_vek, sigma_vek, p_vek, inverted = FALSE)
              {
                #mu_vek = Matrix, deren Spalten Mittelwerte der Komponenten sind
                #sigma_vek = nxnxK Array, element [,,j] ist die j'te Kovarianzmatrix
                #p_vek = Vektor mit Gewichten
                
                leng <- length(mu_vek[1,])
                value <- 0;
                for(i in 1:leng)
                {
                  value = value + p_vek[i]*dnv(x,mu_vek[,i],sigma_vek[,,i],inverted);
                }
                return(value);
              }
              
              
              
              
              zeichnemix_allg <- function(nx=80,ny=80,mu_vek,sigma_vek,p_vek,
                                          sf=1.8,openwin=TRUE, titel = "normal mixture density", 
                                          onescale = FALSE, farbe = "red")
              {
                require(rgl)
                model = "gaussmix"
                
                maxabw_x <- sqrt(max(sigma_vek[1,,]))*sf #maximale Abweichung x
                maxabw_y <- sqrt(max(sigma_vek[2,,]))*sf #maximale Abweichung y
                
                if(onescale)
                {
                  m <- max(maxabw_x, maxabw_y)
                  maxabw_x <- m
                  maxabw_y <- m
                }
                
                links <- min(mu_vek[1,]) - maxabw_x
                rechts <-max(mu_vek[1,]) + maxabw_x
                oben <- max(mu_vek[2,])  + maxabw_y
                unten <- min(mu_vek[2,]) - maxabw_y
                
                #[x(y)start, x(y)end] Intervalle f�r die Werte
                #sx,sy = Anzahl der St�tzstellen pro Intervall (insgsamt sx*sy)
                #openwin = true => es wird f�r jedes bild ein neues fenster ge�ffnet
                
                flag = TRUE
                
                x = seq(links,rechts,length=nx);
                y = seq(unten,oben,length=ny);
                z <- matrix(nrow=nx,ncol=ny)
                
                
                #Gitter erstellen
                for(i in 1:ny)
                {
                  if(model == "gaussmix"){z[,i] <- apply(cbind(x,y[i]),1,gaussmix,mu_vek,sigma_vek,p_vek);}
                  else if(model == "tmix"){z[,i] <- apply(cbind(x,y[i]),1,tmix,mu_vek,sigma_vek,p_vek,freiheitsgrade);}
                  else if(model == "scewmix"){z[,i] <- apply(cbind(x,y[i]),1,scewmix,mu_vek,sigma_vek,p_vek,scew);}    
                }
                if(openwin){open3d()}
                else if(flag){open3d(); flag = FALSE}
                
                
                
                bg3d("white")
                material3d(col="black")
                
                #persp3d(x, y, z, aspect=c(1, 1, 0.5), col = "red", col.mesh = "black",
                #xlab = "X", ylab = "Y", zlab = model,main = paste("mue1 = (",  mu1[1],"," , mu1[2] ,"), mue2=(" , mu2[1],",",mu2[2] , "), p=",p,sep=""), sub = paste(#"K1: ",mat2string(var1), "  K2: ",mat2string(var2)));
                
                
                persp3d(x, y, z, col = farbe, axes=TRUE, main=titel);
                surface3d(x,y,z+0.00002, front="lines", back="culled");
                
                #decorate3d(main = paste("mue1 = (",  mue1[1],"," , mue1[2] ,"), mue2=(" , mue2[1],",",mue2[2] , "), p=",p,sep=""), sub = paste("K1: ",mat2string(var1), " K2: ",mat2string(var2)))
                
              }
                       
              zeichnemix_allg(mu_vek=x@mu,sigma_vek=x@sigma,p_vek=c(x@p,1-sum(x@p)))
              
            }
            
            if(x@dimension>1  && surface==FALSE) {return(warning("Not available yet."))}
            
            if(x@dimension==1){
            
            #parameter umschreiben!
              mu = as.vector(x@mu)
              sigma = as.vector(x@sigma)
              
            getAxis = function(x,...){
              normalMixture=x  
              
              m=length(mu)
              param=c(normalMixture@p[1:(m-1)],mu,sigma)
              
              quantilesMin = NULL
              quantilesMax = NULL
              
              for(i in 1:m){ 
                quantilesMin = c(quantilesMin,qnorm(p=0.0001,mean=mu[i],sd=sigma[i]))
                quantilesMax = c(quantilesMax,qnorm(p=0.9999,mean=mu[i],sd=sigma[i]))                
              }
              
              return(c(min(quantilesMin),max(quantilesMax)))
            }
            
            
            plotDensity = function(l,normalMixture=x,...){
              
              m=length(mu)
              param=c(normalMixture@p[1:(m-1)],mu,sigma)
              
              k = (length(param)+1)/3
              if(k==1){
                return(dnorm(x,mean=param[1],sd=param[2]))
              }
              p   = param[1:(k-1)]
              p[k]= 1-sum(p)
              mu  = param[k:(2*k-1)]
              sig = param[(2*k):(3*k-1)]
              y=numeric(length(l))
              
              for(i in 1:k)
                y= y+p[i]*dnorm(l,mean=mu[i],sd=sig[i])
              
              return(y)
            }
            
            bounds= getAxis(x)
            
            
            if(!missing(add) & !missing(col))
              curve(plotDensity(x),from=bounds[1],to=bounds[2],add=add,col=col,ylab="density")
            if(missing(add) & !missing(col))
              curve(plotDensity(x),from=bounds[1],to=bounds[2],col=col,ylab="density") 
            if(!missing(add) & missing(col))
              curve(plotDensity(x),from=bounds[1],to=bounds[2],add=add,ylab="density")
            if(missing(add) & missing(col))
              curve(plotDensity(x),from=bounds[1],to=bounds[2],ylab="density")
            
            }
            
          })



# C-Version, die ein R-Ergebnis zurueckgibt zum testen
logliDerivatives <- function(object=NULL, parameter=NULL, x, prop = 0, pen = 0, grad = TRUE, hess = TRUE, parallel = TRUE)
{  
  #converting a symmetric or lower diagonal matrix to D*(D+1)/2 Vector
  mat2vec <- function(mat)
  {
    len <- ncol(mat)
    erg <- numeric(0.5*len*(len+1)) 
    for(i in 1:len)
    {
      erg[(0.5*i*(i-1)+1):(0.5*i*(i+1))] <- mat[i,1:i]
    }
    return(erg)
  }
  
  convert_object <- function(object)
  {
    mu_vek <- object@mu
    sigma_vek <- object@sigma
    p_vek <- object@p
    K <- length(mu_vek[1,])
    Dim <- length(mu_vek[,1])
    anz_par <- K*Dim + K*choose(Dim+1,2) + K-1 # Anzahl der Parameter
    startpunkt <- numeric(anz_par)
    
    for(i in 1:K)
    {
      startpunkt[(1 + Dim*(i-1)) : (Dim*i)] <- mu_vek[,i]
      startpunkt[(K*Dim + 1 + (i-1)*Dim*(Dim+1)*0.5):(K*Dim + i*Dim*(Dim+1)*0.5)] <- mat2vec(t(chol(solve(sigma_vek[,,i]))))
    } 
    
    P = array(dim = c(K-1, K-1))
    for(i in 1:(K-1)){P[i,] <- p_vek[i];P[i,i] <- p_vek[i] - 1;}
    q_start <- try(sqrt(solve(P, -p_vek[1:(K-1)]))); 
    if(class(q_start) == "try-error") return("keinpunkt")
    startpunkt[(K*(Dim + 0.5*(Dim+1)*Dim) + 1):(K*(Dim + 0.5*(Dim+1)*Dim) + K - 1)] <- q_start;  
    return(startpunkt)
  }
  
  
  c2r <- function(c_result, K, Dim, prop = 0)
  {
    
    if(!prop)   # falls kein proprtionaler Fall
    {
      grad_len <- K*(Dim + 0.5*Dim*(Dim+1)) + K - 1 
      hess_len <- 0.5*grad_len*(grad_len+1)
    }
    
    else
    {
      grad_len <- K*Dim + 0.5*(Dim+1)*Dim + 2*K - 2
      hess_len <- 0.5*grad_len*(grad_len+1)  
    }
    
    gradient <- c_result[1:(grad_len)]
    hessian <- vec2mat(c_result[(grad_len+1):(grad_len + hess_len)])
    
    hessian <- hessian + t(hessian)*t(lower.tri(hessian))
    
    fval <- c_result[grad_len + hess_len + 1]
    return(list("loglikelihood" = fval, "gradient" = gradient, "hessian" = hessian))
  }
  
  vec2mat <- function(vect, symm = FALSE)
  {
    if(!symm)
    {
      len <- length(vect)
      di = (-1 + sqrt(1+8*len))/2
      erg <- matrix(0, ncol=di, nrow =di)
      for(i in 1:di){for(j in 1:i){erg[i,j] <- vect[i*(i-1)/2 + j]}}
    }
    else
    {
      len <- length(vect)
      di = (-1 + sqrt(1+8*len))/2
      erg <- matrix(0, ncol=di, nrow =di)
      for(i in 1:di){for(j in 1:i){erg[i,j] <- vect[i*(i-1)/2 + j]}}
      erg_c <- t(erg)
      diag(erg_c) <- 0
      erg <- erg + erg_c
    }
    return(erg)
  }
                  
  
  if(is.null(object)&&is.null(parameter)) {stop("Please specify either object or parameter.")}
  
  if(is.null(parameter)){    
    parameter = as.vector(convert_object(object))
  }
  
  
  
  Dim <- ncol(x)
  N <- nrow(x)
  xvec <- c(t(x))
  len <- length(parameter)
  K <- (2*len + 2)/(3*Dim + Dim^2 + 2) # Komponentenanzahl
  output_len = len*(len+1)*0.5 + len + 1;
  the_output = numeric(output_len)
  
  if(!pen) cova = 0
  else cova = mat2vec(cov(x))


if(!parallel)
{
  
  erg_c <- .C("logli_derivatives",
              parameter = as.double(parameter),
              x = as.double(c(t(x))),
              nrowp = as.integer(N),
              Dimp = as.integer(Dim),
              Kp = as.integer(K),
              gradp = as.integer(grad),
              hessp = as.integer(hess),
              prop = as.integer(prop),
              penalizep = as.double(pen),
              cov = as.double(cova),
              the_output  = as.double(the_output))

}


else
{
  erg_c <- .C("logli_derivatives_par",
              parameter = as.double(parameter),
              x = as.double(c(t(x))),
              nrowp = as.integer(N),
              Dimp = as.integer(Dim),
              Kp = as.integer(K),
              gradp = as.integer(grad),
              hessp = as.integer(hess),
              prop = as.integer(prop),
              penalizep = as.double(pen),
              cov = as.double(cova),
              the_output  = as.double(the_output))

}

  erg <- c2r(erg_c$the_output, K, Dim)
  return(erg)
}           





maxAposteriori = function(x,object,detail=FALSE,plot=TRUE,levels=NULL){
  
  if(object@dimension==1){
    
    #find the maximum a posteriori estimates for the components given a dataset 
    #and a ONE-DIMENSIONAL normal mixture model
    maxAposterioriONE = function(sigma,mu,p,k,x,detail=detail){
      
      if(length(p)!=k) p=c(p,1-sum(p))
      
      #state dependent density values 
      W = p[1] * dnorm(x,mean=mu[1],sd=sigma[1])
      for(i in 2:k) W = cbind(W,p[i]*dnorm(x,mean=mu[i],sd=sigma[i]))
      
      #a posteriori's
      W_aposteriori = W/apply(W,1,sum)
      classification = apply(W_aposteriori,1,which.max)
      
      #print(p)
      
      if(detail==TRUE)
        return(list(W_aposteriori=W_aposteriori,classification=classification))
      else
        return(list(classification=classification))
    }

        
    mu=as.vector(object@mu)
    sigma=as.vector(object@sigma)   
    k=length(mu)
    p=object@p[1:(k-1)] 
    
    returnValue = maxAposterioriONE(sigma=sigma,mu=mu,p=p,k=k,x=x,detail=detail)
    
    
    if(plot){
      
      plot(density(x))
      points(x, rep(0,length(x)), col = returnValue$classification, pch = 19, cex = 0.7)
      
    }
    
    return(returnValue)
    
  }
  
  if(object@dimension>1){
    
    maxAposterioriMULT = function(object,x,detail=detail){
     
      k=length(object@mu[1,])
      
      #state dependent density values 
      W = NULL
      for(i in 1:k){ 
        W = cbind(W,object@p[i] * dmvnorm(x=x,mean=object@mu[,i],sigma=object@sigma[,,i]))}
                           
      #a posteriori's
      W_aposteriori = W/apply(W,1,sum)
      classification = apply(W_aposteriori,1,which.max)
      
      if(detail==TRUE)
        return(list(W_aposteriori=W_aposteriori,classification=classification))
      else
        return(list(classification=classification))
      
    }
    
    returnValue =maxAposterioriMULT(object,x,detail)
    
    k=length(object@mu[1,])
    
    
    if(plot && object@dimension==2 && !is.null(levels)){      
      plot(x,pch=19,cex=0.7,col="grey") 
      
        for(j in 1:length(levels))
          for(i in 1:k)
            lines(ellipse(object@sigma[,,i],centre = as.vector(object@mu[,i]), level=levels[j]),col=i,lwd=1)

      
    }
    
    if(plot && object@dimension==2 && is.null(levels)){      
      plot(x,pch=19,cex=0.7,col=returnValue$classification)     
    }
    
    return(returnValue)
    
  }
  
}

convert_object <- function(object)
{
  mu_vek <- object@mu
  sigma_vek <- object@sigma
  p_vek <- object@p
  K <- length(mu_vek[1,])
  Dim <- length(mu_vek[,1])
  anz_par <- K*Dim + K*choose(Dim+1,2) + K-1 # Anzahl der Parameter
  startpunkt <- numeric(anz_par)
  
  for(i in 1:K)
  {
    startpunkt[(1 + Dim*(i-1)) : (Dim*i)] <- mu_vek[,i]
    startpunkt[(K*Dim + 1 + (i-1)*Dim*(Dim+1)*0.5):(K*Dim + i*Dim*(Dim+1)*0.5)] <- mat2vec(t(chol(solve(sigma_vek[,,i]))))
  } 
  
  P = array(dim = c(K-1, K-1))
  for(i in 1:(K-1)){P[i,] <- p_vek[i];P[i,i] <- p_vek[i] - 1;}
  q_start <- try(sqrt(solve(P, -p_vek[1:(K-1)]))); 
  if(class(q_start) == "try-error") return("keinpunkt")
  startpunkt[(K*(Dim + 0.5*(Dim+1)*Dim) + 1):(K*(Dim + 0.5*(Dim+1)*Dim) + K - 1)] <- q_start;  
  return(startpunkt)
}

#converting a symmetric or lower diagonal matrix to D*(D+1)/2 Vector
mat2vec <- function(mat)
{
  len <- ncol(mat)
  erg <- numeric(0.5*len*(len+1)) 
  for(i in 1:len)
  {
    erg[(0.5*i*(i-1)+1):(0.5*i*(i+1))] <- mat[i,1:i]
  }
  return(erg)
}
