\name{convert_object}
\alias{convert_object}
\title{
Converts an object of the class normalMixtureModel into a vector.
}
\description{
Converts an object of the class \code{normalMixtureModel} into a vector (\eqn{\mu_1,\ldots,\mu_k,L_1,\ldots,L_k,q_1,\ldots, q_{k-1}}),
where \eqn{L_i {L_i}^{T} = \Sigma_i^{-1}} and \eqn{p_i = \frac{q_i}{q_1 + \ldots + q_{k-1} + 1}.}
The lower triagonal matrices \eqn{L_i} are vectrorized row wise (see \code{\link{mat2vec}}).
}
\usage{
convert_object(object)
}

\arguments{
  \item{object}{
An object of the class \code{\link{normalMixtureModel-class}}.
}
}

\value{
A vector with length \eqn{k(D+D(D+1)/2) + k - 1}
}

\examples{
# define means and covariaces of the components of a two-dimensional two-component mixture.

# means 
mu = cbind(c(1,1),c(3,3))

# covariances
sigma = array(dim=c(2,2,2))
sigma[,,1] = 0.2*c(1,0.8,0.8,1)
sigma[,,2] = 0.4*c(1,-0.5,-0.5,1)


# Create a normalMixtureModel object
object <- createNormalMixtureModel(p = 0.7, mu = mu, sigma = sigma)

# simulate 700 points from the mixture
x <- simulate(object = object, nsim = 700) 

# estimate the parameters of the mixture from the simulated sample
estimate <- gaussianMixtureMLE(x = x, k = 2, penSig = 1)

# extract the parameters from the returned object and convert them into a vector
pars <- convert_object(estimate$estimatedModel)
}