\name{gaussianMixtureMLE}
\alias{gaussianMixtureMLE}
\title{
estimating (penalized) MLE
}
\description{
This function estimates the parameters of a (multivariate) k-component normal mixture using a penalized maximum likelihood estimator.  
}
\usage{
gaussianMixtureMLE(x, k, object, penP = 0, penSig = 0, doFI = FALSE, tol_eps = 1e-11,
 tol_delta = 1e-11, tol_grad = 1e-11, tol_rlc = 1e-08, tol_em = 1e-06, verbose = FALSE, 
 bcl = 35, maxit = 10, hard_conv = FALSE, parallel = TRUE)
}
\arguments{
  \item{x}{
the dataset, which should be a vector in case of one dimensional fitting and a matrix in the multivariate case
}
  \item{k}{
number of mixture components
}
  \item{object}{
optionally an object of type \code{\link{normalMixtureModel-class}}, its parameters will be used as starting points for the optimization
}
  \item{penP}{
non-negative penalty constant for the weights (default is 0, i.e. no penalization)
}
  \item{penSig}{
non-negative penalty constant for the standard deviations or resp. covariance matrix (default is 0, i.e. no penalization)
}
  \item{doFI}{
     If TRUE, an estimate of the Fisher information matrix of the MLE will be returned (only in the multidimensional case).
}
  \item{tol_eps}{
    Tolerance for the solver of the linear equation systems. A number with absolute value less than \code{tol_eps} is considered as zero (only in the multidimensional case).
}
  \item{tol_delta}{
      Tolerance for a stopping criterion.  If the 2-norm of the Newton's direction is less then \code{tol_delta}, the function returns the current value \eqn{\theta_k} (only in the multidimensional case).
}
  \item{tol_grad}{
      Tolerance for a stopping criterion.  If the 2-norm of the gradient of the log-likelihood is less then \code{tol_grad}, the function returns the current value \eqn{\theta_k} (only in the multidimensional case).
}
  \item{tol_rlc}{
      Tolerance for a stopping criterion.  If the relative log-likelihood change is less then \code{tol_rlc}, the function returns the current value \eqn{\theta_k} (only in the multidimensional case).
}
  \item{tol_em}{
      Tolerance for the stopping criterion for the preceding EM algorithm from the package Mclust. If the relative log-likelihood change during the EM iterations is less than tol_em, than the
      current value \eqn{\theta_k} is beeing passed to the Newton's iteration (only in the multidimensional case).
}
  \item{verbose}{
    If TRUE, some additional outputs will be produced (only for multidimensional case). Default value is FALSE.
}
  \item{bcl}{
     Backtracking length. Maximal number of iterations of the backtracking routine during the line search.
}
  \item{maxit}{
     The maximal number of iterations for the Newton's method.
}
  \item{hard_conv}{
      If TRUE, the algorithm iterates until all stopping criteria are fulfilled. Default value is FALSE.
}
  \item{parallel}{
      If TRUE, some parts of the calculation are carried out in parallel using OpenMP interface.
}

}
\details{

One dimensional case:\cr

In detail the objective function is not only the log-likelihood, but the sum of the log-likelihood, a penalty function depending on the sigmas and a penalty function depending on the weights, i.e.

\deqn{\mathop{loglike}(\mu_1,\ldots,\mu_k,\sigma_1,\ldots,\sigma_k, p_1,\ldots,p_{k-1}|X_1,\ldots,X_n) + c_s \cdot \mathop{pen_1}(\sigma_1,\ldots,\sigma_k,x)+ c_p \cdot \mathop{pen_2}(p_1,\ldots,p_k),}{log-like(mu_1,\ldots,mu_k,sigma_1,\ldots,sigma_k,p_1,\ldots,p_{k-1}|X_1,\ldots,X_n) + c_s * pen_1(sigma_1,\ldots,sigma_k)+ c_p * pen_2(p_1,\ldots,p_k),}

where \eqn{c_s}{c_s} and \eqn{c_p}{c_p} are non-negative constants. These constants determine how strong small values of the parameters should be penalized and thus avoided. 
The penalty functions are given by
\deqn{pen_1(\sigma_1,\ldots,\sigma_k,x) = - \sum_{i=1}^k \frac{s_n^2}{\sigma_i^2} + \log( \frac{s_n^2}{\sigma_i^2})}{ pen_1(\sigma_1,\ldots,\sigma_k,x) = - \sum_{i=1}^k s_n^2/\sigma_i^2 + \log(s_n^2/\sigma_i^2), }
where \eqn{s_n^2}{s_n^2} is the empirical variance, and 
\deqn{pen_2(p_1,\ldots,p_k) = \sum_{i=1}^k \log(p_i).}{pen_2(p_1,\ldots,p_k) = \log(p_i). }
Choosing \eqn{c_s=c_p=0}{c_s=c_p=0} yields the MLE and is the default option. 
If no starting points are supplied, then they are calculated using package \code{mclust}. Thus, if no penalization is used, the used starting point is already the MLE and is only slightly changed.\cr

Multidimensional case:\cr

The function also estimates the parameter of a multivariate k-component normal mixture by maximizing the penalized log-likelihood function: 

\deqn{\mathop{loglike}(\mu_1,\ldots,\mu_k,\Sigma_1,\ldots,\Sigma_k,q_1,\ldots,q_{k-1}|X_1,\ldots,X_n) + c \cdot \mathop{pen}(\Sigma_1, \ldots, \Sigma_k,x).}{loglike(\mu_1,\ldots,\mu_k,\Sigma_1,\ldots,\Sigma_k,q_1,\ldots,q_{k-1}|X_1,\ldots,X_n) + c * pen(\Sigma_1, \ldots, \Sigma_k,x).}

The penalty function is given by
\deqn{pen(\Sigma_1, \ldots, \Sigma_k,x) = - \sum_{i=1}^{k} tr(S_x \Sigma_i^{-1}) + \log|\Sigma_i|}
Where c is a non-negative constant. In contrary to the one dimensional case only the covariances can be penalized.


In the multidimensional case the optimization is carried out with the exact Newton's method. 
If no starting points are supplied, then they are calculated using k-means and the EM-Algorithm. Due to the fact that Newton's method converges locally it is better to supply no starting point rather than a bad starting point.
The function uses internally the following parameterization

\deqn{\mathop{loglike}(\mu_1,\ldots,\mu_k,L_1,\ldots,L_k,q_1,\ldots,q_{k-1}|X_1,\ldots,X_n) + c \cdot \mathop{pen}(L_1, \ldots, L_k),}{loglike(\mu_1,\ldots,\mu_k,L_1,\ldots,L_k,q_1,\ldots,q_{k-1}|X_1,\ldots,X_n) + c * pen(L_1, \ldots, L_k),}
 where \eqn{L_i {L_i}^{T} = \Sigma_i^{-1}}
and \eqn{p_i = \frac{q_i^2}{q_1^2 + \ldots + q_{k-1}^2 + 1}}{p_i = q_i^2/(q_1^2 + \ldots + q_{k-1}^2 + 1)}.

The function uses analytical derivatives.
}
\value{
In the one-dimensional case a list with 4 entries: 
\item{estimatedModel}{the estimated model, which is of the type \code{\link{normalMixtureModel-class}}}
\item{loglik}{A vector with two components. First component: value of the log-likelihood, second component: value of the penalized log-likelihood.}
\item{AIC}{value of the aic}
\item{BIC}{value of the bic}
\cr
In the multidimensional case a list with 6 entries: 
\item{estimatedModel}{the estimated model, which is of the type \code{\link{normalMixtureModel-class}}}
\item{BIC}{value of the bic}
\item{loglik}{A vector with two components. First component: value of the log-likelihood, second component: value of the penalized log-likelihood.}
\item{numit}{A vector with two components. First component: number of EM-iterations to find a starting point, second component: number of Newton's iterations.}
\item{convergence}{A String. Describes which stopping rule took effect.}
\item{MLE_covariance}{An estimate of the covariance matrix of the MLE (- inverse of the Fisher Information), if demanded.}
}
\references{
Chen, J. and Tan,X. "Inference for Multivariate Normal Mixtures", Journal of Multivariate Analysis. (2009)        \cr
Chen, J. and Li, P., "Hypothesis Test for Normal Mixture Models: The EM approach", The Annals of Statistics. (2009)  \cr
Grigory Alexandrovich. An exact Newton's method for ML estimation in a penalized Gaussian mixture model.
}
\author{
Grigory Alexandrovich, Florian Schwaiger
}
\examples{
#one dimensional case
m1 <- createNormalMixtureModel(p = c(0.3,0.4), mu = c(1,3,3.5), sigma = c(0.8,0.8,0.8))
plot(m1)
x <- simulate(object = m1, nsim = 250)
gaussianMixtureMLE(x = x, k = 3, object = m1)$estimatedModel
gaussianMixtureMLE(x = x, k = 3, penSig = 1, penP = 1, object = m1)$estimatedModel

#2 dimensional case
sigma <- array(dim=c(2,2,2))
sigma[,,1] <- 0.2*c(1,0.8,0.8,1)
sigma[,,2] <- 0.4*c(1,-0.5,-0.5,1)
m2 <- createNormalMixtureModel(p = 0.7, mu = cbind(c(1,1),c(3,3)), sigma = sigma)
plot(m2)
x <- simulate(object = m2, nsim = 700)
estimate <- gaussianMixtureMLE(x = x, k = 2, penSig = 1)
}


