\name{maxAposteriori}
\alias{maxAposteriori}

\title{
maximum a posteriori estimates
}
\description{
Find the maximum a posteriori estimates for all data points given a normal mixture model.
}
\usage{
maxAposteriori(x,object,detail = FALSE,plot = TRUE,levels = NULL)
}

\arguments{
  \item{x}{a vector resp. matrix containing the dataset}
  \item{object}{normal mixture model which should be used, see \code{\link{normalMixtureModel-class}} or \code{\link{createNormalMixtureModel}}, possibly an estimated model using \code{\link{gaussianMixtureMLE}}  }
  \item{detail}{when detail equals TRUE also the maximum a posteriori probabilities are returned}
  \item{plot}{If TRUE and datadimension is 1 or 2 a plot will be produced.}
  \item{levels}{If plot = TRUE, optionally a vector with entries in (0,1). Then the contours of the according levels are ploted (see function ellipse from package ellipse).}
}
\value{
Depending on the input value of detail, either the a posteriori clustering or also the a posteriori probabilities.
}

\examples{
#one dimensional case
m1 = createNormalMixtureModel(p = c(0.5), mu = c(1,3), sigma = c(0.8,0.8))
x1 = simulate(object = m1, nsim = 250)
fit1 = gaussianMixtureMLE(x = x1, k = 2, penSig = 1, penP = 1, object = m1)$estimatedModel
clust1 = maxAposteriori(object = fit1, x = x1, detail = FALSE, plot = TRUE)


#2 dimensional case
sigma <- array(dim=c(2,2,2))
sigma[,,1] <- 0.2*c(1,0.8,0.8,1)
sigma[,,2] <- 0.4*c(1,-0.5,-0.5,1)
m2 <- createNormalMixtureModel(p = 0.7, mu = cbind(c(1,1),c(3,3)), sigma = sigma)
x2 <- simulate(object = m2, nsim = 700)
fit2 = gaussianMixtureMLE(x = x2, k = 2, penSig = 1, object = m2)$estimatedModel
#first plot
maxAposteriori(object = fit2, x = x2, detail = FALSE, plot = TRUE)
#second plot
maxAposteriori(object = fit2, x = x2, detail = TRUE,levels=c(0.4,0.9))

}
