\name{pGME-package}
\alias{pGME-package}
\alias{pGME}
\docType{package}
\title{
Estimates the parameters of finite normal mixtures by penalized likelihood methods (with the exact Newton's method in the multivariate case).
}
\description{
Estimating the parameters of finite normal mixtures can lead to difficulties (especially for small sample sizes), if each component of the mixture has possibly a different mean and standard deviation resp. covariance matrix, since then the likelihood is unbounded for any standard deviation parameter going to zero. Further, when estimating a mixture with too many components the parameters are not identifiable anymore since weight parameters can converge towards zero. Both mentioned cases can be avoided by using penalty functions in the log-likelihood. \cr
The package provides functions for penalized maximum likelihood estimation of finite normal mixtures. Due to penalization the algorithm avoids singularities (i.e. prevents any variance to converges towads zero, where the likelihood is unbounded). Further, the penalization can provide a better fit when components are not well separated.
}
\details{
\tabular{ll}{
Package: \tab pGME\cr
Type: \tab Package\cr
Version: \tab 1.0\cr
Date: \tab 2012-08-10\cr
License: \tab GPL-2\cr
Depends: \tab methods,mclust,mvtnorm,rgl,ellipse\cr
}
Given a dataset one can use the function \code{\link{gaussianMixtureMLE}} to estimate the (penalized) MLE. The output is (among other values) an object of the class \code{\link{normalMixtureModel-class}}, which can for example be ploted with the generic plot function \code{\link[pGME]{plot-methods}}. Further, \code{\link[pGME]{simulate-methods}} simulates a dataset from a given mixture object.
Finally, a maximum a posteriori clustering can be computed using the function \code{\link{maxAposteriori}}.
}
\author{
Grigory Alexandrovich, Florian Schwaiger\cr
Maintainer: Grigory Alexandrovich <alexandrovich@mathematik.uni-marburg.de>
}
\references{
Alexandrovich, G., "An exact Newton's method for ML estimation in a penalized Gaussian mixture model". Preprint. (2012)  \cr
Chen, J. and Tan,X. "Inference for Multivariate Normal Mixtures", Journal of Multivariate Analysis. (2009)        \cr
Chen, J. and Li, P., "Hypothesis Test for Normal Mixture Models: The EM approach", The Annals of Statistics. (2009)  \cr
Fraley, C. and Raftery A. "MCLUST Version 3 for R: Normal Mixture Modeling and Model-Based Clustering". (2007)           \cr
Vollmer, S., Holzmann, H. and Schwaiger, F., "Peaks vs. Components." To appear in: Review of Development Economics. (2012)
}

\keyword{package, gaussian mixture, mle, penalized, newton algorithm, unbounded likelihood}

\seealso{
\code{\link{gaussianMixtureMLE}}, \code{\link{logliDerivatives}}, \code{\link[pGME]{plot-methods}}, \code{\link{plotComponents}}, \code{\link[pGME]{simulate-methods}}, \code{\link{maxAposteriori}}
}

\examples{
#one dimensional case
m1 <- createNormalMixtureModel(p = c(0.3,0.4), mu = c(1,3,3.5), sigma = c(0.8,0.8,0.8))
plot(m1)
x <- simulate(object = m1, nsim = 250)
gaussianMixtureMLE(x = x, k = 3, object = m1)$estimatedModel
gaussianMixtureMLE(x = x, k = 3, penSig = 1, penP = 1, object = m1)$estimatedModel

#2 dimensional case
sigma <- array(dim=c(2,2,2))
sigma[,,1] <- 0.2*c(1,0.8,0.8,1)
sigma[,,2] <- 0.4*c(1,-0.5,-0.5,1)
m2 <- createNormalMixtureModel(p = 0.7, mu = cbind(c(1,1),c(3,3)), sigma = sigma)
plot(m2)
x <- simulate(object = m2, nsim = 700)
estimate <- gaussianMixtureMLE(x = x, k = 2, penSig = 1)
}
