#include<Rmath.h>
#include<math.h>
#include<stdarg.h>
#include<stdio.h>
#include<stdlib.h>
#include<omp.h>

/*Parallele Version*/


struct my_array
{
  int anz_dim; /*Anzahl der Dimensionen*/
  int * dims;  /*Anzahl der Elmente pro Dimension*/
  double * values; /*Die eigentlichen Werte*/
};


//Produkt der ersten anz ELemente in p
double prod(int * p, int anz)
{
 double erg = 1;
 int i;
 for(i = 0 ; i < anz; i++){erg *= p[i];} 
 return erg;
}


int min(int a, int b)
{
    if(a < b)   return a;
    return b;
}
//Index des Elementes A[i,j] in vec(A)
int k_delta(int i, int j)
    {
     return (int)(0.5*i*(i+1) + j);
     //else return (int)(0.5*j*(j+1) + i);  
    }


//Summe der ersten anz elemente in p

double sum(double * p, int anz)
{
    double erg = 0;
    int i;
    for(i=0; i < anz; i++){erg+=p[i];}
    return erg;
}

// Liefert das Element mit Koordinatentupel aus ...
// ACHTUNG: die Eintr�ge in ... d�rfen die enstprechenden
// Werte in dims nicht �berschreiten
//Bei 3 Indizes [j,i,k] greift man auf k-te Matrix, j'te Spalte und i'te Zeile zu
double get_element(struct my_array * ma, ...)
{
 int i,index = 0;
 int koords[(ma->anz_dim)];
 va_list listPointer;
 va_start(listPointer, ma);
 for(i = 0; i < ma->anz_dim; i++){koords[i] = va_arg(listPointer,int);}
 for(i = 0; i < ma->anz_dim ; i++)
   {    
    index += prod(ma->dims,ma->anz_dim - i - 1)*koords[ma->anz_dim - 1 - i];
   }
   
 double erg  = (ma -> values)[index];

 va_end( listPointer );
 return erg;
}


//add = 0 heisst �berschreibe das Element, add = 1 heisst addiere drauf
void set_element(struct my_array * ma, double val, int add, ...)
{
 int i,index = 0;
 int koords[(ma->anz_dim)];
 va_list listPointer;
 va_start(listPointer, add);
 for(i = 0; i < ma->anz_dim; i++){koords[i] = va_arg(listPointer, int);}
 for(i = 0; i < ma->anz_dim ; i++)
   {    
    index += prod(ma->dims,ma->anz_dim - i - 1)*koords[ma->anz_dim - 1 - i];
   }
if(!add){(ma -> values)[index] = val;}
else{(ma -> values)[index] += val;}
va_end( listPointer );
}


void free_my_array(struct my_array * ma)
{
 free(ma->dims);
 free(ma->values);
}

void init_ma(struct my_array * ma, int anz_dim, ...)
{
 int i;    
 va_list listPointer; va_start(listPointer, anz_dim);
 
 int * dims = (int *)malloc(sizeof(int)*anz_dim);
 for(i = 0; i < anz_dim; i++){dims[i] = va_arg(listPointer, int);} 
 ma->anz_dim = anz_dim; ma->dims = dims;  ma->values = (double *)malloc(sizeof(double)*((int)prod(dims, anz_dim))); 
 va_end(listPointer);
}


void stop()
{    
  exit(1);
}


void zs_delta(int i, double * output)
{
 int z;
 double zz = 0.5 + sqrt(0.25 + 2*i);
 int fl_z = (int)floor(zz);
  
 if(fl_z == zz){z = (int)(zz - 1);}
 
 else z = fl_z; 
 int sp = (int)(i - 0.5*z*(z-1));

 //wegen zero based anpassen
 output[0] = z-1;
 output[1] = sp-1;
}

//Funktion zum Berechnen von vv^TL als Vektor
//v ist s'te Zeile der Matrix V
//erg soll ein double * sein mit len_L speicher
int vvL(struct my_array * V, int s, double * L, int len_L, struct my_array * zs_mat, int Dim, double * erg)
{
  int j,k = 0;     
    
  for(k = 0; k < len_L; k++)
    {
      int zs1 = (int)get_element(zs_mat,0,k); // Nummer der Zeile  
      int zs2 = (int)get_element(zs_mat,1,k); // Nummer der Spalte
      double vL = 0; 
      for(j = zs2; j < Dim; j++){vL += get_element(V,j,s)*L[(int)(0.5*j*(j+1)+zs2)];}
      erg[k] = get_element(V,zs1,s)*vL;
    }
  return 1;
}
   
// Berechnet x^t A_k x
// A_k muss eine Matrix sein
// Erwartet ein Array mit 3 Dimensionen (A[i,j,k])
double billinearform(double * x, struct my_array * A, int k)
{
 if((A->dims)[0] != (A->dims)[1]) stop();
 //short int * koords = malloc(sizeof(short int)*3);
 double erg = 0;
 int i,j;
 int Dim = (A->dims)[0]; //Anzahl Spalten soll gleich Anzahl der Zeilen sein

 for(j = 0; j < (Dim-1); j++)
 {
    for(i = j+1; i < Dim; i++)
    {
     erg += 2*x[i]*x[j]*get_element(A,j,i,k);
    }
  erg += x[j]*x[j]*get_element(A,j,j,k); //Diagonalelement
 }
 
 erg += x[Dim-1]*x[Dim-1]*get_element(A,Dim-1,Dim-1,k); //letztes Element aufaddieren
 return erg;
}

// Multiplikation x=Ab. my_array muss eine Matrix beinhalten
int mult_matrix_vektor(struct my_array * A, const double * b, int trans, double * output)
{
 if(A->anz_dim != 2) stop();
 int i,j;
 
 if(!trans)
 {
     for(i=0; i < (A->dims)[1]; i++)
     {
        output[i]  = 0;              
              for(j = 0; j < (A->dims)[0]; j++)
              {
               output[i] +=  get_element(A,j,i)*b[j];
                
              }
     }
 }

 else //falls transponiert
 {
     for(i=0; i < (A->dims)[1]; i++)
     {
        output[i]  = 0;              
              for(j = 0; j < (A->dims)[0]; j++)
              {
               output[i] +=  get_element(A,i,j)*b[j];
                
              }
     }
 }
 return 1;     
}


int d2pdq2(int i, int k, double * parameter, int * q_vec, double nenner_h, int K, double * output)
{
 int l;    
 double nenner3 = nenner_h*nenner_h*nenner_h;
 double temp_sum = 1; for(l=0; l < K - 1; l++){if(l != i) temp_sum += parameter[q_vec[l]] * parameter[q_vec[l]];}
 
 if(i != k)
 {
  for(l=0; l < K-1; l++)
  {
   if(l != i && l != k) output[l] = (8*parameter[q_vec[i]]*parameter[q_vec[k]]*parameter[q_vec[l]]*parameter[q_vec[l]])/nenner3;
  }  
   output[i] = 4*parameter[q_vec[i]]*parameter[q_vec[k]]*(nenner_h - 2*temp_sum)/nenner3;
   output[k] = 4*parameter[q_vec[i]]*parameter[q_vec[k]]*(2*parameter[q_vec[k]]*parameter[q_vec[k]] - nenner_h)/nenner3;
 }
 
 else
 {
   for(l=0; l < K-1; l++){output[l] = 2*parameter[q_vec[l]]*parameter[q_vec[l]]*(4*parameter[q_vec[i]]*parameter[q_vec[i]] - nenner_h)/nenner3;}
   output[i] = 2*temp_sum*(nenner_h - 4*parameter[q_vec[i]]*parameter[q_vec[i]])/nenner3;
 }
 
 return 1; 
}


void mixture_derivatives( int Dim,
                          int K,  
                          double * parameter,
                          double * x,
                          int gradient,
                          int hessian,
                          struct my_array * zsmat, 
                          struct my_array * SigmaInv, 
                          double * detL,
                          struct my_array * dpdq,
                          double * sum_dpdq,
                          struct my_array * d2pdq2, 
                          double   nenner, 
                          double   nenner2, 
                          double * p_vec,
                          int output_len,
                          double * the_output)
{
 int muu,muu2,Lu,Lu2,pind,i,j,k,l,m,stelle,zs1,zs2,zs_ind1,zs_ind2,f=0; //indizes f�r innere Schleifen
 int gew_ind =(K*(Dim + 0.5*(Dim+1)*Dim)); //Index von die Gewichte angangen
 double pec;
 int len = K*(0.5*Dim*(Dim+1) + Dim) + K - 1; // L�nge des Parametervektors
 double grad[len]; //Vektor f�r den aktuellen Gradient dg(x_i)/dtheta, (in output wir gleich ableitung der loglikelihood geschrieben)
 int len_L =  0.5*Dim*(Dim+1);
 double spr,con = pow(M_2PI,-0.5*Dim);// Normierungskonstante double exps = 0;
 double Q[Dim]; //Ableitungen der quadratischen Formen nach mu
 struct my_array M;
 init_ma(&M, 2, Dim, Dim);
 double QQ[len_L]; //vvTL als Vektor
 double fval = 0; // Wert der Loglikelihood
 double dgdq[K-1];
 double ddet[len_L];
 double dddet[len_L];
 //double temp_sum[K-1]; //F�r Summen der Spalten aus dpdq
 for(i = 0; i < len_L; i++){ddet[i]=dddet[i]=0;}
 //for(i = 0; i < K-1; i++){temp_sum[i]=0;}
 // Jetzt Ableitungen dg/dp dp/dq_i
 // exps, detL, dfdp, dpdq, Funktionswert Berechnen
 double exps[K];
 double dgdp[K];
 double phi[K];
 double val,val2,val3; //platzhalter f�r operationen
 
 //K xdiffs
 struct my_array xdiff;
 init_ma(&xdiff,2,Dim,K);
 double temp[Dim]; //tempor�re Variable zum speichern von xdiff der aktuellen Komponente
 
 // Einige Gr��en berechnen, sie werden sp�ter ben�tigt
 for(j = 0; j < K; j++)
  {    
   //xdiff[,j] berechnen
   for(i=0; i < Dim; i++){temp[i]= x[i] -  parameter[Dim*j + i]; set_element(&xdiff, temp[i],0, i,j);}
   exps[j] = billinearform(temp, SigmaInv, j); exps[j] = exp(-0.5*exps[j]);
   phi[j] = detL[j]*con*exps[j];
   fval = fval + p_vec[j]*detL[j]*con*exps[j];    
  }
  
  if(fval < 1e-20) return;
  
  
  //funktionswert speichern  
  the_output[output_len - 1] += log(fval);
  
  if(!gradient) return;
  
   
  for(i=0 ; i < K-1 ; i++){dgdp[i] = phi[i] - phi[K-1];}
  
  //dgdq berechnen: dgdq <- t(dgdp) %*% dpdq  
  mult_matrix_vektor(dpdq, dgdp, 1, dgdq); //dpdq^T dgdp
  //dgdq reinschreiben
  

  //letzte Zeile von output ist Gradient
  for(i = 0; i < K-1; i++)
        {
             stelle = (int)(K*(Dim + 0.5*(Dim+1)*Dim) + i);
             grad[stelle] = dgdq[i];
             the_output[stelle]+= dgdq[i]/fval;     
        }

  
 //Jetzt Schleife �ber Komponenten
 for(i = 0; i < K; i++)
 {
      //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      //~~~~~~~~~~~~~~~~~~~~~~~~ GRADIENT ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
      
      muu = Dim*i;
      pind = (int) K*(Dim + 0.5*(Dim+1)*Dim) + i;
      Lu  = (int) (K*Dim + i*Dim*(Dim+1)*0.5); //erster Index von L_i in parameter (untere Grenze)
      pec = p_vec[i]*exps[i]*con;
      // xdiff existiert schon als my_array
      
      //Q <- SigmaInv[,,i] %*% xdiff :
      for(j = 0; j < Dim; j++)
      {
        Q[j] = 0;
        for(k = 0; k < Dim; k++)
        {                            //k'te Spalte, j'te Zeile    
         Q[j] += get_element(SigmaInv, k,j,i)*get_element(&xdiff,k,i);
        }  
      }
      
      
      //M <- xdiff %*% t(xdiff);
      for(j = 0; j < Dim; j++)
      {
            for(k = 0; k <= j; k++)
            {   
             set_element(&M, get_element(&xdiff,j,i)*get_element(&xdiff,k,i), 0, k,j);
             if(j != k) set_element(&M, get_element(&xdiff,j,i)*get_element(&xdiff,k,i), 0, j, k);
            }
      }
      
         //QQ <- vvL(xdiff,parameter[Lu:Lo])       
         f = vvL(&xdiff, i,  &parameter[Lu], len_L, zsmat, Dim, QQ);
         
         if(f != 1) stop();

         f = 0;

         //grad[muu:muo] <- p_vec[i]*detL[i]*con*exps[i]*Q
         for(j = 0; j < Dim; j++){val = detL[i]*pec*Q[j]; the_output[muu + j] += val/fval; grad[muu + j] = val;}
         
         //Ableitung  dgdL 
         //for(ii in 1:Dim){ddet[0.5*ii*(ii+1)] <- detL[i]/parameter[(K*Dim + (i-1)*Dim*(Dim+1)*0.5)  + 0.5*ii*(ii-1) + ii]};
         for(j = 0; j < Dim; j++){ddet[(int)(0.5*(j+1)*(j+2) - 1)] = detL[i]/parameter[(int)(Lu + 0.5*(j+1)*(j+2) - 1)];}

         //grad[Lu:Lo] <- p_vec[i]*exps[i]*con*(ddet - detL[i]*QQ)
         for(j=0; j < len_L; j++){val = pec*(ddet[j] - detL[i]*QQ[j]); the_output[Lu + j] += val/fval; grad[Lu+j]= val;}
         
     
     if(!hessian){continue;}
     
     //.....................................................//
     //----------------- HESSE MATRIX ---------------------//
     //.....................................................//
 
     //d2pdq2[i,j,] %*% dgdp berechnen
    val = 0;

                                                          
    if(i < K - 1)
     {
          for(k = 0; k <= i ; k ++)    
          {
           //d2pdq2[j,i,k] ist dp_j/dq_i dq_k
           for(j = 0; j < K-1 ; j++){val += get_element(d2pdq2,j,k,i)*dgdp[j];} 
           l = (int)(K*(Dim + 0.5*(Dim+1)*Dim) + k);
           m = (int)(K*(Dim + 0.5*(Dim+1)*Dim) + i);
           val = (val - (1./fval)*grad[l]*grad[m])/fval;
           
           the_output[len + k_delta(m,l)] += val;
           //set_element(output, val, 1, l, m); //hier wird kummuliert
           val = 0;
           //print_myarray(output,1);
          }
     }
     
     
     val  = 0;

      //dgdmudmu
     for(j=0; j < Dim; j++)
     {
        for(k = 0; k <= j; k++)
        {
         val = pec*detL[i]*(Q[j]*Q[k] - get_element(SigmaInv,k,j,i));
         val = (val - (1./fval)*grad[muu + k]*grad[muu + j])/fval;         
         the_output[len + k_delta(muu+j,muu+k)] += val;
         //set_element(output, val, 1, muu+k, muu+j);    
        }
     }
            
     val  = 0;
     

    if(i < K-1) 
    {
      for(j = 0; j < K-1; j++) //Schleife �ber q_1,...,q_K-1
      {
        for(l=0; l < Dim; l++) //Schleife �ber mu_i1,...,mu_iDim
        {        
          val = exps[i]*con*detL[i]*Q[l]*get_element(dpdq,j,i);
          val = (val - (1./fval)*grad[muu + l]*grad[len - K + 1 + j])/fval;
          the_output[len + k_delta(len - K + 1 + j, muu + l)] += val;
        }           
      }
     
     }
    
    else
    {
      for(j = 0; j < K-1; j++)
        {             
          //for(m = 0; m < K-1; m++){temp_sum[j] += get_element(dpdq,j,m);}
          for(l = 0; l < Dim; l++)
          {
           val = -exps[i]*con*detL[i]*Q[l]*sum_dpdq[j];
           val = (val - (1./fval)*grad[muu + l]*grad[len - K + 1 + j])/fval;
           the_output[len + k_delta(gew_ind + j, muu + l)] += val;
          }
        }
    }
    
        
     //dLdmu, dLdL
 for(j = 0; j < len_L; j++)
 {
   val = 0; val2 = 0; val3 = 0; spr = 0;
   zs1 = (int)get_element(zsmat,0,j); // Nummer der Zeile  
   zs2 = (int)get_element(zsmat,1,j); // Nummer der Spalte
   
            
//dLdmu  ACHTUNG k_delta(i,j) nur f�r i >= j anwenden
          
          for(m = zs2; m < Dim; m++){spr += (parameter[Lu + k_delta(m,zs2)]*get_element(&xdiff,m,i));}
          
          val3 = -detL[i]*QQ[j]; 
          if(zs1 == zs2){val3 = val3 + detL[i]/parameter[Lu + j];}
                                  
          for(l = 0; l < Dim; l++)
          {
                             
             if(l != zs1){val = pec*(val3*Q[l] + ((l>=zs2)?parameter[Lu + k_delta(l,zs2)]*detL[i]*get_element(&xdiff,zs1,i):0));}  

             else
             {
               //s2[zs[1]] <- (L[,zs[2]] %*% xdiff + xdiff[zs[1]]*L[zs[1],zs[2]])*detL[i];
               val2 = (spr + get_element(&xdiff,zs1,i)*parameter[Lu + j])*detL[i];
               val = pec*(val3*Q[l] + val2);
             } 
             val = (val - (1./fval)*grad[muu + l]*grad[Lu + j])/fval;
             the_output[len + k_delta(Lu+j, muu + l)] += val;           
             val2 = 0;
         }        
             val3 = 0; val = 0;
 //dLdL 
    val = detL[i]*QQ[j];
    if(zs1 == zs2)
           {
             val = val - detL[i]/parameter[Lu + j];
             for(l = 0; l < Dim; l++){dddet[(int)(0.5*(l+1)*(l+2) - 1)] = ddet[(int)(0.5*(l+1)*(l+2) - 1)]/parameter[Lu + j];}
             dddet[j] = 0;
           }
           
    
   for(l = 0; l <= j ; l++)
          {
            val2  = 0;
            val3 = 0;  
            
            zs_ind1 = (int)get_element(zsmat,0,l); // Nummer der Zeile  
            zs_ind2 = (int)get_element(zsmat,1,l); // Nummer der Spalte 
            
            if(zs_ind1 == zs_ind2){val2 = QQ[j]*detL[i]/parameter[Lu + l];}
            
            if(zs_ind2 == zs2){val2 += detL[i]*get_element(&M,zs_ind1,zs1);}
            
            val3 = pec*(val*QQ[l] + ((zs1==zs2)?dddet[l]:0) - val2);
            
            val3 = (val3 - (1./fval)*grad[Lu + l]*grad[Lu + j])/fval; 
            
            the_output[len + k_delta(Lu+j,Lu+l)] += val3;  
        
          }                  
 } //Ende Schleife �ber Elemente von L

 //dLdq

  val = 0;
        
  if(i < K-1)
  {
       for(m = 0; m < K-1; m++)
       {
             for(l = 0; l < len_L; l++)
             {
                   val = exps[i]*con*(ddet[l] - detL[i]*QQ[l])*get_element(dpdq,m,i);
                   val = (val - (1./fval)*grad[Lu + l]*grad[gew_ind + m])/fval; 
                   the_output[len + k_delta(gew_ind+m,Lu+l)] += val;
             }
       
       } 
  }
   
  else
  {
    for(m = 0; m < K-1; m++)  
    {
          for(l = 0; l < len_L; l++)
          {
                val = -exps[i]*con*(ddet[l] - detL[i]*QQ[l])*sum_dpdq[m];
                val = (val - (1./fval)*grad[Lu + l]*grad[gew_ind + m])/fval;
                the_output[len + k_delta(gew_ind+m,Lu+l)] += val; 
          }
    }
      
  }
}
  //ENDE Schleife �ber Komponenten
  
if(hessian)
{
  //Jetzt Elemente dmu_i dmu_j f�r i != j und dmu_i dSigma_j
  
    for(i=0; i < K-1; i++)
  {
      muu = Dim*i;
      Lu = (int) (K*Dim + i*Dim*(Dim+1)*0.5); 
                  
           for(j = i+1; j < K; j++)
           {
               muu2 = Dim*j;
               Lu2 = (int) (K*Dim + j*Dim*(Dim+1)*0.5);
               
               for(l=0; l < Dim; l++)
               {
                        for(m = 0; m < Dim; m++)
                        {
                          val = (- (1./fval)*grad[muu + l]*grad[muu2 + m])/fval;    
                          the_output[len + k_delta(muu2+m,muu+l)] += val;     
                        }
                        
                        
                        for(m = 0; m < len_L; m++)
                        {
                          val = (- (1./fval)*grad[Lu2 + m]*grad[muu + l])/fval;
                          the_output[len + k_delta(Lu2+m,muu+l)] += val;            
                        }
               }
               
               
             
              for(l = 0; l < len_L; l++) 
              {
                    
                        for(m = 0; m < Dim; m++)
                        {
                          val = (- (1./fval)*grad[Lu + l]*grad[muu2 + m])/fval;    
                          the_output[len + k_delta(Lu+l, muu2+m)] += val;     
                        }       
                    
                        for(m = 0; m < len_L; m++)
                        {
                          val = (- (1./fval)*grad[Lu2 + m]*grad[Lu + l])/fval;
                          the_output[len + k_delta(Lu2+m,Lu+l)] += val;       
                        }
              } 
           }
  }
}  


   free_my_array(&M);
   free_my_array(&xdiff);
}// Ende Programm



/******************* PROPORTIONALER FALL*****************/
//Model g(x) = SUM_i=1..K p_i phi(x; mu_i, L lambda_(i-1))
// parameter = mu_1,...,mu_K,L,lambda1,...,lambda_K-1,q_1,...,q_K-1

void mixture_derivatives_prop( int Dim,
                          int K,  
                          double * parameter,
                          double * x,
                          int gradient,
                          int hessian,
                          struct my_array * zsmat, 
                          struct my_array * SigmaInv, 
                          double * detL,
                          double * ddet,
                          struct my_array * dpdq,
                          double * sum_dpdq,
                          struct my_array * d2pdq2, 
                          double   nenner, 
                          double   nenner2, 
                          double * p_vec,
                          int output_len,
                          double * the_output)                          
{
 int muu,muu2,Lu,pind,lind,lambdau,i,j,k,l,m,stelle,zs1,zs2,zs_ind1,zs_ind2,f=0; //indizes f�r innere Schleifen
 int gew_ind = (K*Dim + 0.5*(Dim+1)*Dim  + K - 1 ); //Index wo die Gewichte anfangen
 double pec;
 int len = K*Dim + 0.5*Dim*(Dim+1) + 2*K - 2; // L�nge des Parametervektors
 double grad[len]; //Vektor f�r den aktuellen Gradient dg(x_i)/dtheta, (in output wir gleich ableitung der loglikelihood geschrieben)
 for(i = 0; i < len; i++){grad[i] = 0;}
 int len_L = 0.5*Dim*(Dim+1);
 double tLt, spr,con = pow(M_2PI,-0.5*Dim);// Normierungskonstante double exps = 0;
 double Q[K][Dim]; //Ableitungen der quadratischen Formen nach mu
 struct my_array M;
 init_ma(&M, 2, Dim, Dim);
 double QQ[K][len_L]; //vvTL als Vektor
 double fval = 0; // Wert der Loglikelihood
 double dgdq[K-1];
 //double ddet[len_L];
 double lambda[K]; //poportionskonstanten lambda[0] = 1;
 double lambda2[K];
 double lambdaD[K];
 double outputL_temp[(int)(len_L*(len_L + 1)*0.5)]; // F�r dLdL, da erst �ber alle Komponenten summiert werden muss
 double dLdq_temp[len_L][K-1]; //F�r dLdq
 
 for(i = 0; i < len_L; i++)
 {
       for(j = 0; j < K -1; j++)
       {
         dLdq_temp[i][j] = 0;
       }
 }
 
 for(i= 0 ; i < (int)(len_L*(len_L + 1)*0.5); i++){outputL_temp[i] = 0;}
 
 double dddet[len_L];
 //double temp_sum[K-1]; //F�r Summen der Spalten aus dpdq
 for(i = 0; i < len_L; i++){dddet[i]=0;}
 //for(i = 0; i < K-1; i++){temp_sum[i]=0;}
 // Jetzt Ableitungen dg/dp dp/dq_i
 // exps, detL, dfdp, dpdq, Funktionswert Berechnen
 double bills[K]; //F�r Billinearformen
 double exps[K];
 double dgdp[K];
 double phi[K];
 double val,val2,val3; //platzhalter f�r operationen
 Lu  = K*Dim; //erster Index von L in parameter (untere Grenze)
 
 //K xdiffs
 struct my_array xdiff;
 init_ma(&xdiff,2,Dim,K);
 double temp[Dim]; //tempor�re Variable zum speichern von xdiff der aktuellen Komponente
 
 lambda[0] = 1;
 lambda2[0] = 1;
 lambdaD[0] = 1;
 
 lambdau = (int)(K*Dim + 0.5*Dim*(Dim+1));
 for(j = 1; j < K; j++)
  {
   lambda[j] = fabs(parameter[lambdau+j-1]);
   lambda2[j] = lambda[j]*lambda[j];
   lambdaD[j] = pow(lambda[j],Dim);
  }
 
 // Einige Gr��en berechnen, sie werden sp�ter ben�tigt
 for(j = 0; j < K; j++)
  {
   //xdiff[,j] berechnen
   for(i=0; i < Dim; i++){temp[i] = x[i] -  parameter[Dim*j + i]; set_element(&xdiff,temp[i],0, i,j);}
   bills[j] = billinearform(temp, SigmaInv, 0); 
   exps[j] = exp(-0.5*lambda2[j]*bills[j]);
   phi[j] = lambdaD[j]*detL[0]*con*exps[j];
   fval = fval + p_vec[j]*lambdaD[j]*detL[0]*con*exps[j];
  }
  
  if(fval < 1e-20) return;
 
  //funktionswert speichern  
  the_output[output_len - 1] += log(fval);
  
  if(!gradient && !hessian) return;
  
  for(i=0 ; i < K-1 ; i++){dgdp[i] = phi[i] - phi[K-1];}
  
  //dgdq berechnen: dgdq <- t(dgdp) %*% dpdq  
  mult_matrix_vektor(dpdq, dgdp, 1, dgdq); //dpdq^T dgdp
  //dgdq reinschreiben
  
  //letzte Zeile von output ist Gradient
  for(i = 0; i < K-1; i++)
   {
    stelle = (int)(K*Dim + 0.5*(Dim+1)*Dim + K - 1 + i);
    grad[stelle] = dgdq[i];
    the_output[stelle]+= dgdq[i]/fval;     
   }  
   
   
  //Jetzt Schleife �ber Komponenten
 for(i = 0; i < K; i++)
 {      
      //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      //~~~~~~~~~~~~~~~~~~~~~~~~ GRADIENT ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
      
      muu = Dim*i;
      pec = p_vec[i]*exps[i]*con;
      
      // xdiff existiert schon als my_array
      
      //Q <- lambda2*SigmaInv %*% xdiff :
      for(j = 0; j < Dim; j++)
      {
        Q[i][j] = 0;
        for(k = 0; k < Dim; k++)
        {                            //k'te Spalte, j'te Zeile    
         Q[i][j] += lambda2[i]*get_element(SigmaInv, k, j, 0)*get_element(&xdiff,k,i);
        }
      }
      
      
         //QQ <- vvL(xdiff,parameter[Lu:Lo])
         //Achtung, hier ist im Unterschied zu der R Version noch kein Faktor lambda^2 dranmultipliziert
         f = vvL(&xdiff, i,  &parameter[Lu], len_L, zsmat, Dim, QQ[i]);

         
         if(f != 1) stop();
         
         f = 0;
         
       //Ableitung dgdL 

      //grad[Lu:Lo] <- p_vec[i]*exps[i]*con*(ddet - detL[i]*QQ)
      for(j=0; j < len_L; j++){val = lambdaD[i]*pec*(ddet[j] - detL[0]*lambda2[i]*QQ[i][j]); the_output[Lu + j] += val/fval; grad[Lu+j] += val;}
                 
 }
 
 
 for(i = 0; i < K; i++)
 {
      muu = Dim*i;
      pind = (int) (K*Dim + 0.5*(Dim+1)*Dim + K - 1);
      lind = (int) (K*Dim + 0.5*(Dim+1)*Dim);
      pec = p_vec[i]*exps[i]*con;
      tLt = 0;
      
      //tLt <- (t(xdiff) %*% Q)[1]
      for(j = 0; j < Dim; j++){tLt += get_element(&xdiff,j,i)*Q[i][j];}
      
     //M <- xdiff %*% t(xdiff);
      for(j = 0; j < Dim; j++)
      {
            for(k = 0; k <= j; k++)
            {   
             set_element(&M, get_element(&xdiff,j,i)*get_element(&xdiff,k,i), 0, k,j);
             if(j != k) set_element(&M, get_element(&xdiff,j,i)*get_element(&xdiff,k,i), 0, j, k);
            }
      }
         
         //grad[muu:muo] <- p_vec[i]*detL[i]*con*exps[i]*Q
         for(j = 0; j < Dim; j++){val = lambdaD[i]*detL[0]*pec*Q[i][j]; the_output[muu + j] += val/fval; grad[muu + j] = val; }
                    
         // dgDlambda
         if(i > 0)
         {
          val = p_vec[i]*con*detL[0]*pow(fabs(lambda[i]),(Dim-1))*exps[i]*(Dim - lambda2[i]*bills[i]);
          the_output[lambdau+i-1] += val/fval;
          grad[lambdau+i-1]= val;
         }         
         
      
        if(!hessian){continue;}
             
                                 
         //.....................................................//
         //----------------- HESSE MATRIX ---------------------//
         //.....................................................// 
 val  = 0;
 if(i > 0) //falls i = 1..K-1
 {
   //dmudlambda
   for(j = 0; j < Dim; j++)
   {
         val = pec*lambdaD[i]*detL[0]*Q[i][j]/lambda[i]*(Dim + 2 - tLt);
         val = (val - (1./fval)*grad[muu + j]*grad[lind + i-1])/fval;
         the_output[len + k_delta(lind+i-1, muu+j)] += val;
         
   }
   
   val = 0;
 
   //#### dlambdadlambda ####
   //the_hessian[lambdau, lambdau] <- p_vec[i]*lambda^(Dim-2)*con*detL*exps[i]*((Dim-1 - tLt)*(Dim - tLt) - 2*tLt)  
   val = pec*pow(lambda[i], Dim - 2)*detL[0]*((Dim - 1 - tLt)*(Dim - tLt) - 2*tLt);
   val = (val - (1./fval)*grad[lind+i-1]*grad[lind+i-1])/fval;
   the_output[len + k_delta(lind+i-1, lind+i-1)] += val;  
   
 
   val = 0;
  
if(i > 0)
{  
   //dlambda_i dq
   if(i == K-1)
    {     
         for(j = 0; j < K-1; j++)
         {
                val =  -sum_dpdq[j]*con*detL[0]*pow(lambda[i], Dim - 1)*exps[i]*(Dim - tLt);
                val = (val - (1./fval)*grad[gew_ind+j]*grad[lind+i-1])/fval;                
                the_output[len + k_delta(gew_ind+j,lind+i-1)] += val;
         }            
    }
    
    else
    {
         for(j = 0; j < K-1; j++)
         {
                val =  get_element(dpdq,j,i)*con*detL[0]*pow(lambda[i], Dim - 1)*exps[i]*(Dim - tLt);
                val = (val - (1./fval)*grad[lind+i-1]*grad[gew_ind+j])/fval; 
                the_output[len + k_delta(gew_ind+j,lind+i-1)] += val;
         }
    }
    
   val = 0;
}
   
 /*        
      #### dLdlambda ####
      the_hessian[lambdau, Lu:Lo] <- p_vec[i]*con*lambdaD/lambda*exps[i]*((Dim - tLt)*(ddet - detL*QQ) + lambda*(- 2*detL*QQ/lambda))
         
 */
 
 for(j = 0; j < len_L; j++)
       {
                val = pec*lambdaD[i]/lambda[i]*((Dim - tLt)*(ddet[j] - detL[0]*lambda2[i]*QQ[i][j]) - lambda[i]*(2*detL[0]*lambda2[i]*QQ[i][j]/lambda[i]));                
                val = (val - (1./fval)*grad[lind+i-1]*grad[Lu + j])/fval;
                the_output[len + k_delta(lind+i-1,Lu+j)] += val;
       }           
 }
         
         
 //d2pdq2[i,j,] %*% dgdp berechnen
          
 val = 0;
 
    //dg2dq2                                                      
  if(i < K-1)
     {
          for(k = 0; k <= i ; k ++)    
          {
           //d2pdq2[j,i,k] ist dp_j/dq_i dq_k
           for(j = 0; j < K-1 ; j++){val += get_element(d2pdq2,j,k,i)*dgdp[j];} 
           l = (int)(K*Dim + 0.5*(Dim+1)*Dim + K - 1 + k);
           m = (int)(K*Dim + 0.5*(Dim+1)*Dim + K - 1 + i);
           val = (val - (1./fval)*grad[l]*grad[m])/fval;
           
           the_output[len + k_delta(m,l)] += val;
           //set_element(output, val, 1, l, m); //hier wird kummuliert
           val = 0;
           //print_myarray(output,1);
          }
     }
     
     
     //dLdq_i
 val = 0;
        
  if(i < K-1)
  {
       for(m = 0; m < K-1; m++)
       {
             for(l = 0; l < len_L; l++)
             {
                   val = lambdaD[i]*exps[i]*con*(ddet[l] - lambda2[i]*detL[0]*QQ[i][l])*get_element(dpdq,m,i);
                   dLdq_temp[l][m] += val;
             }
       
       } 
  }
 
  else
  {
        for(m = 0; m < K-1; m++)  
        {
              for(l = 0; l < len_L; l++)
              {
                    val = -lambdaD[i]*exps[i]*con*(ddet[l] - lambda2[i]*detL[0]*QQ[i][l])*sum_dpdq[m];
                    dLdq_temp[l][m] += val;                    
                    //val = (val - (1./fval)*grad[Lu + l]*grad[gew_ind + m])/fval;
                    //the_output[len + k_delta(gew_ind+m,Lu+l)] += val;
              }
        }
      
  }
     
   val  = 0;
     
      //dgdmudmu 
    for(j = 0; j < Dim; j++)
     {
       for(k = 0; k <= j; k++)
        {
         val = pec*detL[0]*lambdaD[i]*(Q[i][j]*Q[i][k] - lambda2[i]*get_element(SigmaInv,k,j,0));
         val = (val - (1./fval)*grad[muu + k]*grad[muu + j])/fval;         
         the_output[len + k_delta(muu+j,muu+k)] += val;
        }
     }
     
     
  val  = 0;
     
    //dmu_idq 
    if(i < K-1) 
    {
      for(j = 0; j < K-1; j++) //Schleife �ber q_1,...,q_K-1
      {
        for(l=0; l < Dim; l++) //Schleife �ber mu_i1,...,mu_iDim
        {        
          val = exps[i]*con*detL[0]*lambdaD[i]*Q[i][l]*get_element(dpdq,j,i);
          val = (val - (1./fval)*grad[muu + l]*grad[len - K + 1 + j])/fval;
          the_output[len + k_delta(len - K + 1 + j, muu + l)] += val;
        }
      }
     }
    
    else
    {
      for(j = 0; j < K-1; j++)
        {
          //for(m = 0; m < K-1; m++){temp_sum[j] += get_element(dpdq,j,m);}
          for(l = 0; l < Dim; l++)
          {
           val = -exps[i]*con*lambdaD[i]*detL[0]*Q[i][l]*sum_dpdq[j];
           val = (val - (1./fval)*grad[muu + l]*grad[len - K + 1 + j])/fval;
           the_output[len + k_delta(gew_ind + j, muu + l)] += val;
          }
        }
    }
  
//dLdL, dLdmu
 for(j = 0; j < len_L; j++)
 {
   val = 0; val2 = 0; val3 = 0; spr = 0;
   zs1 = (int)get_element(zsmat,0,j); // Nummer der Zeile  
   zs2 = (int)get_element(zsmat,1,j); // Nummer der Spalte
  
   spr = 0;
   for(m = zs2; m < Dim; m++){spr += (parameter[Lu + k_delta(m,zs2)]*get_element(&xdiff,m,i));}
   
//dLdmu  ACHTUNG k_delta(i,j) nur f�r i >= j anwenden
          

          val3 = - lambda2[i]*detL[0]*QQ[i][j]; 
          if(zs1 == zs2){val3 = val3 + detL[0]/parameter[Lu + j];}
                                  
          for(l = 0; l < Dim; l++)
          {
                         
             if(l != zs1){val = lambdaD[i]*pec*(val3*Q[i][l] + lambda2[i]*((l>=zs2)?parameter[Lu + k_delta(l,zs2)]*detL[0]*get_element(&xdiff,zs1,i):0));} 
             
             else
             {                 
               //s2[zs[1]] <- (L[,zs[2]] %*% xdiff + xdiff[zs[1]]*L[zs[1],zs[2]])*detL[i];
               val2 = lambda2[i]*(spr + get_element(&xdiff,zs1,i)*parameter[Lu + j])*detL[0];
               val = lambdaD[i]*pec*(val3*Q[i][l] + val2);
                            
             }

             val = (val - (1./fval)*grad[muu + l]*grad[Lu + j])/fval;

             
             the_output[len + k_delta(Lu+j, muu + l)] += val;           
             val2 = 0;
          
          }        
             val3 = 0; val = 0;
             


   //dLdL
    val = lambda2[i]*detL[0]*QQ[i][j];
    
    if(zs1 == zs2)
           {
             val = val - detL[0]/parameter[Lu + j];
             for(l = 0; l < Dim; l++){dddet[(int)(0.5*(l+1)*(l+2) - 1)] = ddet[(int)(0.5*(l+1)*(l+2) - 1)]/parameter[Lu + j];}
             dddet[j] = 0;
           }
           
           
   for(l = 0; l <= j ; l++)
          {
            val2  = 0;
            val3 = 0;  
            
            zs_ind1 = (int)get_element(zsmat,0,l); // Nummer der Zeile  
            zs_ind2 = (int)get_element(zsmat,1,l); // Nummer der Spalte 
            
            if(zs_ind1 == zs_ind2)
            {
              val2 = lambda2[i]*QQ[i][j]*detL[0]/parameter[Lu + l];
            }
            
            if(zs_ind2 == zs2){val2 += detL[0]*lambda2[i]*get_element(&M,zs_ind1,zs1);}
            
            val3 = lambdaD[i]*pec*(val*lambda2[i]*QQ[i][l] + ((zs1==zs2)?dddet[l]:0) - val2);
            outputL_temp[k_delta(j,l)] += val3; 
            //the_output[len + k_delta(Lu+j,Lu+l)] += val3;
          }    
     } //Ende Schleife �ber Elemente von L 
    }//Ende Schleide �ber Komponenten
    
    
    //Jetzt die  fehlenden Opeartionen f�r dLdL durchf�hren
  
  if(hessian)
  {
   
    for(j = 0; j < len_L; j++)
    {
        for(l = 0; l <= j; l++)
        {
          the_output[len + k_delta(Lu+j,Lu+l)] += (1./fval * outputL_temp[k_delta(j,l)] - (1./fval)*grad[Lu + l]*grad[Lu + j]/fval);   
        }    
    }
    
    
    for(l = 0; l < len_L; l++)
    {
       for(m = 0; m < K - 1; m++)   
       {      
        the_output[len + k_delta(gew_ind+m,Lu+l)] +=  (dLdq_temp[l][m] - (1./fval)*grad[Lu + l]*grad[gew_ind + m])/fval;
       }   
    }
      
  }
  
if(hessian)
{
  //Jetzt Elemente dmu_i dmu_j f�r i != j und dlambda_i dlambda_j
  lind =(int)(K*Dim + Dim*(Dim+1)*0.5);
  pind = lind + K - 1;
  
  for(i=0; i < K-1; i++)
  {
      muu = Dim*i;
    
           for(j = i+1; j <= K-1; j++)
           {
               muu2 = Dim*j;
               
               
               for(l=0; l < Dim; l++)
               {
                        for(m = 0; m < Dim; m++)
                        {
                          val = (- (1./fval)*grad[muu + l]*grad[muu2 + m])/fval;    
                          the_output[len + k_delta(muu2+m,muu+l)] += val;
                        }
               }
           
               //Ableitung dlambda_i dlambda_j
               if(j < K - 1)
              {
               val = (- (1./fval)*grad[lind + i]*grad[lind + j])/fval;
               the_output[len + k_delta(lind+j , lind+i)] += val; 
              }
           }
         
    }
   
  for(i=0; i < K; i++)//l�uft mu_0,...,mu_K-1 durch
  {       
   muu = Dim*i;
       
      for(j = 1; j < K; j++) //l�uft lambda_1,...,lambda_K-1 durch
      {
            if(j != i)
           {        
                 //Ableitung dlambda_j dmu_i 
                 for(l = 0; l < Dim; l++)
                  {
                      val = (- (1./fval)*grad[muu+l]*grad[lind + j-1])/fval;
                      the_output[len + k_delta(lind+j-1 , muu+l)] += val;                                          
                  }
           }
      }
  }
  
}   
  
   free_my_array(&M);
   free_my_array(&xdiff);    
}

/*******************************************************/
/*************** Ende Proportionaler Fall **************/
/*******************************************************/
//nrow = Anzahl der Zeilen von x
//Dim = Anzahl der Spalten von x = Dimension des Raums
//prop = 0 => allgemeiner Fall prop = 1 => proportionaler Fall

void logli_derivatives_par(double * parameter, 
			   double * x, 
			   int * nrowp, 
			   int * Dimp, 
			   int * Kp, 
			   int * gradp, 
			   int * hessp, 
			   int * prop, 
			   double * penalizep, 
			   double * cov,
			   double * the_output)
{
  int Dim = * Dimp;
  int nrow = * nrowp;
  int K = * Kp;
  int hess = * hessp;
  int grad = * gradp;
  double penalize = * penalizep;
  int kdim = K*Dim; //Länge der Erwartungswerte
  int len_L =  (int)(0.5*Dim*(Dim+1)); //L�nge von L
  double temp_sum;
  int i,j,k,l,s,z;

if(*prop == 0)
{  
      int Lu; //Schleifen indizes, Zeilen,Spalten
      int len = (int)(K*(Dim + 0.5*Dim*(Dim+1)) + K - 1); //L�nge von parameter
      int output_len = (int)len*(len+1)*0.5 + len + 1;
      int q_vec[K-1]; for(i=0; i < K - 1 ; i++){q_vec[i] = (int)(K*(Dim + 0.5*(Dim+1)*Dim) + i);}
      double nenner_h = 1; for(i = 0; i < K - 1; i++){nenner_h += parameter[q_vec[i]]*parameter[q_vec[i]];} //Nenner der Gewichte
      double nenner2_h = nenner_h * nenner_h;
      int index_vec[Dim];for(i = 0; i < Dim; i++){index_vec[i] = (int)(i + 0.5*(i+1)*i);} // Indizes der Diagonalelemente in vec(L)      
     
      //SigmaInv_h <- array(dim = c(Dim,Dim,K))
      struct my_array SigmaInv_h;
      init_ma(&SigmaInv_h,3,Dim,Dim,K);
      double detL_h[K];
      struct my_array dpdq_h;
      init_ma(&dpdq_h,2,K-1,K-1);
      
      struct my_array zs_vals;
      init_ma(&zs_vals,2,2,len_L);
      
      struct my_array d2pdq2_h;
      init_ma(&d2pdq2_h,3,K-1,K-1,K-1); 
                 
      double p_vec[K]; p_vec[K-1] = 1;
      for(i = 0; i < K - 1; i++){p_vec[i] = parameter[q_vec[i]]*parameter[q_vec[i]] / nenner_h; p_vec[K-1] -= p_vec[i];}
      
      // Einige Gr��en berechnen, sie werden sp�ter ben�tigt
      for(j = 0; j < K; j++)
      {
       int Lu  = (int) (K*Dim + j*Dim*(Dim+1)*0.5); //erster Index von L_i in parameter (untere Grenze)          
       if(j < K-1) 
        {     
        for(k=0; k < K-1; k++)if(k!=j){set_element(&dpdq_h,-2*parameter[q_vec[j]]*parameter[q_vec[k]]*parameter[q_vec[k]]/nenner2_h,0,j,k);}
        double temp_sum = 0; for(k = 0; k < K - 1; k++){if(k != j) temp_sum += parameter[q_vec[k]]*parameter[q_vec[k]];}
        set_element(&dpdq_h,  2*parameter[q_vec[j]] * (temp_sum + 1)/nenner2_h,0,j,j);
        }
         
        //detL_h[j] <- prod(parameter[(K*Dim + 1 + (j-1)*Dim*(Dim+1)*0.5):(K*Dim + j*Dim*(Dim+1)*0.5)][index_vec]); # Determinante
        detL_h[j] = 1;
        for(k=0; k < Dim; k++){detL_h[j] *= parameter[(int)(K*Dim + j*Dim*(Dim+1)*0.5) + index_vec[k]];}
        
        //Jetzt SigmaInv Konstruieren
        double val;
        
        for(i = 0; i < Dim; i ++)
        {
              for(k = 0; k <= i; k++)
              {
               //SigmaInv[k,i,j] bef�llen
               val = 0;
               for(l=0; l <= k; l++){val += parameter[(int)(Lu + (0.5*i*(i+1) + l))]*parameter[Lu + (int)(0.5*k*(k+1) + l)];}
               set_element(&SigmaInv_h,val,0,k,i,j);
               if(i > k) set_element(&SigmaInv_h,val,0,i,k,j); 
              }
        }
      }
      
        
      for(k = 0; k < K - 1; k++)
      {
       for(i = 0; i <=k; i++)      
       {
        d2pdq2(i,k,parameter,q_vec,nenner_h,K, &d2pdq2_h.values[i*(K-1) + k*(K-1)*(K-1)]);  
       }      
      }
      
      //zs_vals <- array(dim = c(len_L,2)) 
      //for(i in 1:len_L){zs_vals[i,] <-  zs_delta(i)}
      for(i = 0; i < len_L; i++){zs_delta(i+1, &zs_vals.values[2*i]);}
      
      double * sum_dpdq = malloc(sizeof(double)*K);
      
      
       
      for(j = 0; j < K-1; j++)
        {
          sum_dpdq[j] = 0;
          for(i = 0; i < K-1; i++){sum_dpdq[j] += get_element(&dpdq_h,j,i);}    //Spaltensummen in dpdq
        }
        
     
     int anz_threads;
     #pragma omp parallel
     {if(omp_get_thread_num() == 1)anz_threads = omp_get_num_threads();}
        
        
     // ############# PRALELLISIEREN ###################
     double our_output[anz_threads][output_len];
     
    for(i = 0; i < anz_threads; i++)
    {
	for(j = 0; j < output_len; j++)
	{
	  our_output[i][j] = 0.0;
	}
    }    
     
   
    #pragma omp parallel 
    {       
      #pragma omp for
      //jetzt schleife ueber die Datenpunkte  
      for(i = 0; i < nrow; i++) //Parallelisieren
      {
            
       mixture_derivatives(  Dim,
                             K, 
                             parameter, 
                             &x[i*Dim],
                             grad, 
                             hess, 
                             &zs_vals, 
                             &SigmaInv_h, 
                             detL_h,
                             &dpdq_h, 
                             sum_dpdq, 
                             &d2pdq2_h, 
                             nenner_h, 
                             nenner2_h, 
                             p_vec,
                             output_len, 
                             our_output[omp_get_thread_num()]); 
      }
      
    }
    
    
    
    //Jetzt wieder zusammenführen
    for(i = 0; i < anz_threads; i++)
    {
	for(j = 0; j < output_len; j++)
	{
	  the_output[j] +=our_output[i][j];
	}
    }
    
    // printf("Output: \n");
    // print_output(the_output, output_len, len);
    
    
     the_output[2] = the_output[2]; // wegen dem komischen Bug von .C interface in R, das sonst nur Teile von the_output gef�llt werden ???
     the_output[1] = the_output[1];
     
     
    //#######################################
    //############# Penalisierung ############
    //#######################################
    
 if(penalize > 0)
    {
      //Kovarianzmatrix erstellen
    if(cov[0] == 0)
     {
		double cov[(int)(Dim*(Dim+1)*0.5)];
		double means[Dim];
	
	//Mittelwerte in jeder Dimension ermitteln
	for(i = 0; i < Dim ; i++)
	{
	  means[i] = 0;
	  for(j = 0; j < nrow; j++){means[i] += x[j*Dim + i];}
	  means[i] = means[i]/(double)nrow;
	}
	
	for(i = 0; i < Dim; i++)
	{
	  for(j = 0; j <= i; j++)
	  {
	    cov[k_delta(i,j)] = 0;
	    
	    for(k = 0; k < nrow; k++)
	    {
	    cov[k_delta(i,j)] += (x[k*Dim + i] - means[i]) * (x[k*Dim + j] - means[j]);
	    }
	    cov[k_delta(i,j)] =  cov[k_delta(i,j)]/((double)nrow-1);
	  }
	}
     }
     //Ende Kovarianzmatrix erstellen
       
     
     
  //#####Likelihoodwert-anpassen######
  double spur = 0;
     
     for(i = 0; i < K; i++)
      {
	//Spur(Cov*Sigma_i^-1) berechnen
	for(j = 0; j < Dim; j++)
	{
	  for(k = 0; k < Dim; k++)
	  {
	    
	    for(l = 0; l < j; l++)
	    {
			 //k'te Spalte, j'te Zeile 
	     spur += get_element(&SigmaInv_h, k,j,i)*cov[k_delta(j,l)];
	    }
	    
	    for(l = j; l < Dim; l++)
	    {
	     spur += get_element(&SigmaInv_h, k,j,i)*cov[k_delta(l,j)];
	    } 
	  } 
	}
	
	the_output[output_len - 1] -=  penalize*(spur + log(1./(detL_h[i]*detL_h[i])));
	spur = 0;
      }
     

     //Ableitungen anpassen
      for(i = 0; i < K; i++)  
      {
	Lu = (int) (kdim + i*Dim*(Dim+1)*0.5); 
	//Gradient von Li
	for(j = 0; j < len_L; j++)
	{	
	  temp_sum = 0; 
	  z = (int)get_element(&zs_vals,0,j); // Nummer der Zeile  
	  s = (int)get_element(&zs_vals,1,j); // Nummer der Spalte
	  
	  
	  for(k = s; k < z; k++){temp_sum += cov[k_delta(z,k)]*parameter[kdim + i*len_L + k_delta(k,s)];}
	  for(k = z; k < Dim; k++){temp_sum += cov[k_delta(k,z)]*parameter[kdim + i*len_L + k_delta(k,s)];}
	  
		  
	  if(z == s){temp_sum -= 1/parameter[kdim+ i*len_L + j];}
	  the_output[K*Dim + i*len_L + j] -= 2*penalize*temp_sum;

	  temp_sum = 0;
	  
	  for(l = 0; l <= j; l++)
	    { 
	      int zl = (int)get_element(&zs_vals,0,l); // Nummer der Zeile  
	      int sl = (int)get_element(&zs_vals,1,l); // Nummer der Spalte
	      
	      temp_sum += (sl==s)*cov[k_delta(zl,z)];
	      if(z==s && l==j) temp_sum += 1./(parameter[kdim + i*len_L + j]*parameter[kdim + i*len_L + j]);
	      
	      the_output[len + k_delta(Lu + j, Lu + l)] -= 2*penalize*temp_sum; 
	    }
	 
	}//Ende j
	      
      } 
    }//Ende if penalize
    
    //####################################################################
    //################ Ende falls penalisierung aktiviert ################
    //####################################################################
         
    
     free(sum_dpdq);
     free_my_array(&SigmaInv_h);
     free_my_array(&dpdq_h);
     free_my_array(&d2pdq2_h); 
     free_my_array(&zs_vals);
} // Ende if prop == 0



/*--------------------------*/
/* Falls proportionaler Fall*/
/*--------------------------*/
else
{
     int i,j,k,l; //Schleifen indizes
     int len = (int)(K*Dim + 0.5*Dim*(Dim+1) + 2*K - 2); //L�nge von parameter
     int len_L =  (int)(0.5*Dim*(Dim+1)); //L�nge von L
     int Lu  = K*Dim; //erster Index von L in parameter (untere Grenze) 
     int output_len = (int)len*(len+1)*0.5 + len + 1;
     int q_vec[K-1]; for(i=0; i < K - 1 ; i++){q_vec[i] = (int)(K*Dim + 0.5*(Dim+1)*Dim + K - 1 + i);}
     double nenner_h = 1; for(i = 0; i < K - 1; i++){nenner_h += parameter[q_vec[i]]*parameter[q_vec[i]];} //Nenner der Gewichte
     double nenner2_h = nenner_h * nenner_h;
     int index_vec[Dim];for(i = 0; i < Dim; i++){index_vec[i] = (int)(i + 0.5*(i+1)*i);} // Indizes der Diagonalelemente in vec(L)
   
     //SigmaInv_h <- array(dim = c(Dim,Dim,K))
     struct my_array SigmaInv_h;
     init_ma(&SigmaInv_h,3,Dim,Dim,1);
     
     double detL_h[1];
      
     struct my_array dpdq_h = * (struct my_array *) malloc(sizeof(struct my_array));
     init_ma(&dpdq_h,2,K-1,K-1);
      
     struct my_array zs_vals = * (struct my_array *) malloc(sizeof(struct my_array));
     init_ma(&zs_vals,2,2,len_L);
      
     struct my_array d2pdq2_h = * (struct my_array *) malloc(sizeof(struct my_array));
     init_ma(&d2pdq2_h,3,K-1,K-1,K-1); 
                 
     double p_vec[K]; p_vec[K-1] = 1;
     for(i = 0; i < K - 1; i++){p_vec[i] = parameter[q_vec[i]]*parameter[q_vec[i]] / nenner_h; p_vec[K-1] -= p_vec[i];}
   
     //Jetzt SigmaInv Konstruieren
     double val;
        
     for(i = 0; i < Dim; i ++)
        {
              for(k = 0; k <= i; k++)
              {
               //SigmaInv[k,i,j] bef�llen
               val = 0;
               for(l=0; l <= k; l++){val += parameter[(int)(Lu + (0.5*i*(i+1) + l))]*parameter[Lu + (int)(0.5*k*(k+1) + l)];}
               set_element(&SigmaInv_h,val,0,k,i,0);
               if(i > k) set_element(&SigmaInv_h,val,0,i,k,0); 
              }
        }   
   

   // Einige Groessen berechnen, sie werden sp�ter ben�tigt
   for(j = 0; j < K; j++)
      {         
       if(j < K-1) 
        {     
        for(k=0; k < K-1; k++)if(k!=j){set_element(&dpdq_h,-2*parameter[q_vec[j]]*parameter[q_vec[k]]*parameter[q_vec[k]]/nenner2_h,0,j,k);}
        double temp_sum = 0; for(k = 0; k < K - 1; k++){if(k != j) temp_sum += parameter[q_vec[k]]*parameter[q_vec[k]];}
        set_element(&dpdq_h,  2*parameter[q_vec[j]] * (temp_sum + 1)/nenner2_h,0,j,j);
        }
         
        //detL_h[j] <- prod(parameter[(K*Dim + 1 + (j-1)*Dim*(Dim+1)*0.5):(K*Dim + j*Dim*(Dim+1)*0.5)][index_vec]); # Determinante
        detL_h[0] = 1;
        for(k=0; k < Dim; k++){detL_h[0] *= parameter[(int)(Lu + index_vec[k])];}
       
      }//Ende j
      
   for(k = 0; k < K - 1; k++)
      {
       for(i = 0; i <=k; i++)      
       {
        d2pdq2(i,k,parameter,q_vec,nenner_h,K, &d2pdq2_h.values[i*(K-1) + k*(K-1)*(K-1)]);  
       }
      }
      
       //zs_vals <- array(dim = c(len_L,2)) 
       //for(i in 1:len_L){zs_vals[i,] <-  zs_delta(i)}
       for(i = 0; i < len_L; i++){zs_delta(i+1, &zs_vals.values[2*i]);}
       
       
       
       double * ddet = malloc(sizeof(double)*len_L);
       for(j = 0; j < len_L; j++){ddet[j] = 0;}
       //for(ii in 1:Dim){ddet[0.5*ii*(ii+1)] <- detL[i]/parameter[(K*Dim + (i-1)*Dim*(Dim+1)*0.5)  + 0.5*ii*(ii-1) + ii]};
       for(j = 0; j < Dim; j++){ddet[(int)(0.5*(j+1)*(j+2) - 1)] = detL_h[0]/parameter[(int)(Lu + 0.5*(j+1)*(j+2) - 1)];}
       
       
       double * sum_dpdq = malloc(sizeof(double)*K);
       
      for(j = 0; j < K-1; j++)
        {
          sum_dpdq[j] = 0;
          for(i = 0; i < K-1; i++){sum_dpdq[j] += get_element(&dpdq_h,j,i);}
          
        }
      
      
      
     int anz_threads;
     #pragma omp parallel
     {if(omp_get_thread_num() == 1)anz_threads = omp_get_num_threads();}  
     
    double our_output[anz_threads][output_len]; 
    
    #pragma omp parallel 
    {       
      #pragma omp for
      //jetzt schleife �ber die Datenpunkte  
      for(i = 0; i < nrow; i++) //Parallelisieren
      {
            
       mixture_derivatives_prop(Dim,
                             K, 
                             parameter, 
                             &x[i*Dim],
                             grad, 
                             hess, 
                             &zs_vals, 
                             &SigmaInv_h, 
                             detL_h,
                             ddet,
                             &dpdq_h, 
                             sum_dpdq,
                             &d2pdq2_h, 
                             nenner_h, 
                             nenner2_h, 
                             p_vec,
                             output_len, 
                             the_output); 
      }
      
    }
  
    //Jetzt wieder zusammenführen
    for(i = 0; i < anz_threads; i++)
    {
	for(j = 0; j < output_len; j++)
	{
	  the_output[j] +=our_output[i][j];
	}
    }  
  
    
    
     the_output[2] = the_output[2]; // wegen dem komischen Bug von .C interface in R, das sonst nur Teile von the_output gef�llt werden ???
    
     free(sum_dpdq);
     free(ddet);
     free_my_array(&SigmaInv_h);
     free_my_array(&dpdq_h);
     free_my_array(&d2pdq2_h); 
     free_my_array(&zs_vals);
          
}//Ende if(prop)
 
  
}//Ende Funktion






/*Seriell*/
void logli_derivatives(double * parameter,
		       double * x,
		       int * nrowp,
		       int * Dimp,
		       int * Kp,
		       int * gradp, 
		       int * hessp,
		       int * prop,
		       double * penalizep,
		       double * cov,
		       double * the_output)
{
  int Dim = * Dimp;
  int nrow = * nrowp;
  int K = * Kp;
  int hess = * hessp;
  int grad = * gradp;
  double penalize = * penalizep;
  int kdim = K*Dim; //Länge der Erwartungswerte
  int len_L =  (int)(0.5*Dim*(Dim+1)); //L�nge von L
  double temp_sum;
  int i,j,k,l,s,z;
  
  
if(*prop == 0)
{
      int Lu; //Schleifen indizes, Zeilen,Spalten
      int len = (int)(K*(Dim + 0.5*Dim*(Dim+1)) + K - 1); //L�nge von parameter
      int output_len = (int)len*(len+1)*0.5 + len + 1;
      int q_vec[K-1]; for(i=0; i < K - 1 ; i++){q_vec[i] = (int)(K*(Dim + 0.5*(Dim+1)*Dim) + i);}
      double nenner_h = 1; for(i = 0; i < K - 1; i++){nenner_h += parameter[q_vec[i]]*parameter[q_vec[i]];} //Nenner der Gewichte
      double nenner2_h = nenner_h * nenner_h;
      int index_vec[Dim];for(i = 0; i < Dim; i++){index_vec[i] = (int)(i + 0.5*(i+1)*i);} // Indizes der Diagonalelemente in vec(L)
          
      //SigmaInv_h <- array(dim = c(Dim,Dim,K))
      struct my_array SigmaInv_h;
      init_ma(&SigmaInv_h,3,Dim,Dim,K);
      double detL_h[K];
      struct my_array dpdq_h;
      init_ma(&dpdq_h,2,K-1,K-1);
      
      struct my_array zs_vals;
      init_ma(&zs_vals,2,2,len_L);
      
      struct my_array d2pdq2_h;
      init_ma(&d2pdq2_h,3,K-1,K-1,K-1); 
                 
      double p_vec[K]; p_vec[K-1] = 1;
      for(i = 0; i < K - 1; i++){p_vec[i] = parameter[q_vec[i]]*parameter[q_vec[i]] / nenner_h; p_vec[K-1] -= p_vec[i];}
      
      // Einige Gr��en berechnen, sie werden sp�ter ben�tigt
      for(j = 0; j < K; j++)
      {
       int Lu  = (int) (K*Dim + j*Dim*(Dim+1)*0.5); //erster Index von L_i in parameter (untere Grenze)          
       if(j < K-1) 
        {     
        for(k=0; k < K-1; k++)if(k!=j){set_element(&dpdq_h,-2*parameter[q_vec[j]]*parameter[q_vec[k]]*parameter[q_vec[k]]/nenner2_h,0,j,k);}
        temp_sum = 0; for(k = 0; k < K - 1; k++){if(k != j) temp_sum += parameter[q_vec[k]]*parameter[q_vec[k]];}
        set_element(&dpdq_h,  2*parameter[q_vec[j]] * (temp_sum + 1)/nenner2_h,0,j,j);
        }
         
        //detL_h[j] <- prod(parameter[(K*Dim + 1 + (j-1)*Dim*(Dim+1)*0.5):(K*Dim + j*Dim*(Dim+1)*0.5)][index_vec]); # Determinante
        detL_h[j] = 1;
        for(k=0; k < Dim; k++){detL_h[j] *= parameter[(int)(K*Dim + j*Dim*(Dim+1)*0.5) + index_vec[k]];}
        
        //Jetzt SigmaInv Konstruieren
        double val;
        
        for(i = 0; i < Dim; i ++)
        {
              for(k = 0; k <= i; k++)
              {
               //SigmaInv[k,i,j] bef�llen
               val = 0;
               for(l=0; l <= k; l++){val += parameter[(int)(Lu + (0.5*i*(i+1) + l))]*parameter[Lu + (int)(0.5*k*(k+1) + l)];}
               set_element(&SigmaInv_h,val,0,k,i,j);
               if(i > k) set_element(&SigmaInv_h,val,0,i,k,j); 
              }
        }
      }
      
        
      for(k = 0; k < K - 1; k++)
      {
       for(i = 0; i <=k; i++)      
       {
        d2pdq2(i,k,parameter,q_vec,nenner_h,K, &d2pdq2_h.values[i*(K-1) + k*(K-1)*(K-1)]);  
       }      
      }
      
      //f <- array(dim = c(len_L,2)) 
      //for(i in 1:len_L){zs_vals[i,] <-  zs_delta(i)}
      for(i = 0; i < len_L; i++){zs_delta(i+1, &zs_vals.values[2*i]);}
      
      double * sum_dpdq = malloc(sizeof(double)*K);
      
      
       
      for(j = 0; j < K-1; j++)
        {
          sum_dpdq[j] = 0;
          for(i = 0; i < K-1; i++){sum_dpdq[j] += get_element(&dpdq_h,j,i);}    //Spaltensummen in dpdq
        }
      
            
      //jetzt schleife �ber die Datenpunkte  
      for(i = 0; i < nrow; i++) //Parallelisieren
      {
            
       mixture_derivatives(  Dim,
                             K, 
                             parameter, 
                             &x[i*Dim],
                             grad, 
                             hess, 
                             &zs_vals, 
                             &SigmaInv_h, 
                             detL_h,
                             &dpdq_h, 
                             sum_dpdq, 
                             &d2pdq2_h, 
                             nenner_h, 
                             nenner2_h, 
                             p_vec,
                             output_len, 
                             the_output); 
      }
      
      
    // printf("Output: \n");
    // print_output(the_output, output_len, len);
    
    //#######################################
    //############# Penalisierung ############
    //#######################################
    
    if(penalize > 0)
    {
      //Kovarianzmatrix erstellen
    if(cov[0] == 0)
     {
		double cov[(int)(Dim*(Dim+1)*0.5)];
		double means[Dim];
	
	//Mittelwerte in jeder Dimension ermitteln
	for(i = 0; i < Dim ; i++)
	{
	  means[i] = 0;
	  for(j = 0; j < nrow; j++){means[i] += x[j*Dim + i];}
	  means[i] = means[i]/(double)nrow;
	}
	
	for(i = 0; i < Dim; i++)
	{
	  for(j = 0; j <= i; j++)
	  {
	    cov[k_delta(i,j)] = 0;
	    
	    for(k = 0; k < nrow; k++)
	    {
	    cov[k_delta(i,j)] += (x[k*Dim + i] - means[i]) * (x[k*Dim + j] - means[j]);
	    }
	    cov[k_delta(i,j)] =  cov[k_delta(i,j)]/((double)nrow-1);
	  }
	}
     }
     //Ende Kovarianzmatrix erstellen
       
     
     
     //#####Likelihoodwert-anpassen######
     double spur = 0;
     
     for(i = 0; i < K; i++)
      {
	//Spur(Cov*Sigma_i^-1) berechnen
	for(j = 0; j < Dim; j++)
	{
	  for(k = 0; k < Dim; k++)
	  {
	    
	    for(l = 0; l < j; l++)
	    {
			 //k'te Spalte, j'te Zeile 
	     spur += get_element(&SigmaInv_h, k,j,i)*cov[k_delta(j,l)];
	    }
	    
	    for(l = j; l < Dim; l++)
	    {
	     spur += get_element(&SigmaInv_h, k,j,i)*cov[k_delta(l,j)];
	    } 
	  } 
	}
	
	the_output[output_len - 1] -=  penalize*(spur + log(1./(detL_h[i]*detL_h[i])));
	spur = 0;
      }
     

     //Ableitungen anpassen
      for(i = 0; i < K; i++)  
      {
	Lu = (int) (kdim + i*Dim*(Dim+1)*0.5); 
	//Gradient von Li
	for(j = 0; j < len_L; j++)
	{	
	  temp_sum = 0; 
	  z = (int)get_element(&zs_vals,0,j); // Nummer der Zeile  
	  s = (int)get_element(&zs_vals,1,j); // Nummer der Spalte
	  
	  
	  for(k = s; k < z; k++){temp_sum += cov[k_delta(z,k)]*parameter[kdim + i*len_L + k_delta(k,s)];}
	  for(k = z; k < Dim; k++){temp_sum += cov[k_delta(k,z)]*parameter[kdim + i*len_L + k_delta(k,s)];}
	  
		  
	  if(z == s){temp_sum -= 1/parameter[kdim+ i*len_L + j];}
	  the_output[K*Dim + i*len_L + j] -= 2*penalize*temp_sum;

	  temp_sum = 0;
	  
	  for(l = 0; l <= j; l++)
	    { 
	      int zl = (int)get_element(&zs_vals,0,l); // Nummer der Zeile  
	      int sl = (int)get_element(&zs_vals,1,l); // Nummer der Spalte
	      
	      temp_sum += (sl==s)*cov[k_delta(zl,z)];
	      if(z==s && l==j) temp_sum += 1./(parameter[kdim + i*len_L + j]*parameter[kdim + i*len_L + j]);
	      
	      the_output[len + k_delta(Lu + j, Lu + l)] -= 2*penalize*temp_sum; 
	    }
	 
	}//Ende j
	      
      } 
    }//Ende if penalize
    
    //####################################################################
    //################ Ende falls penalisierung aktiviert ################
    //####################################################################
    
    
     the_output[2] = the_output[2]; // wegen dem komischen Bug von .C interface in R, das sonst nur Teile von the_output gef�llt werden ???
    
     free(sum_dpdq);
     free_my_array(&SigmaInv_h);
     free_my_array(&dpdq_h);
     free_my_array(&d2pdq2_h); 
     free_my_array(&zs_vals);
} // Ende if prop == 0



/*--------------------------*/
/* Falls proportionaler Fall*/
/*--------------------------*/
else
{
     int i,j,k,l; //Schleifen indizes
     int len = (int)(K*Dim + 0.5*Dim*(Dim+1) + 2*K - 2); //L�nge von parameter
     int output_len = (int)len*(len+1)*0.5 + len + 1;
     int q_vec[K-1]; for(i=0; i < K - 1 ; i++){q_vec[i] = (int)(K*Dim + 0.5*(Dim+1)*Dim + K - 1 + i);}
     double nenner_h = 1; for(i = 0; i < K - 1; i++){nenner_h += parameter[q_vec[i]]*parameter[q_vec[i]];} //Nenner der Gewichte
     double nenner2_h = nenner_h * nenner_h;
     int index_vec[Dim];for(i = 0; i < Dim; i++){index_vec[i] = (int)(i + 0.5*(i+1)*i);} // Indizes der Diagonalelemente in vec(L)
   
     //SigmaInv_h <- array(dim = c(Dim,Dim,K))
     struct my_array SigmaInv_h;
     init_ma(&SigmaInv_h,3,Dim,Dim,1);
     
     double detL_h[1];
      
     struct my_array dpdq_h;
     init_ma(&dpdq_h,2,K-1,K-1);
      
     struct my_array zs_vals;
     init_ma(&zs_vals,2,2,len_L);
      
     struct my_array d2pdq2_h;
     init_ma(&d2pdq2_h,3,K-1,K-1,K-1); 
                 
     double p_vec[K]; p_vec[K-1] = 1;
     for(i = 0; i < K - 1; i++){p_vec[i] = parameter[q_vec[i]]*parameter[q_vec[i]] / nenner_h; p_vec[K-1] -= p_vec[i];}
   
     //Jetzt SigmaInv Konstruieren
     double val;
        
     for(i = 0; i < Dim; i ++)
        {
              for(k = 0; k <= i; k++)
              {
               //SigmaInv[k,i,j] bef�llen
               val = 0;
               for(l=0; l <= k; l++){val += parameter[(int)(kdim + (0.5*i*(i+1) + l))]*parameter[kdim + (int)(0.5*k*(k+1) + l)];}
               set_element(&SigmaInv_h,val,0,k,i,0);
               if(i > k) set_element(&SigmaInv_h,val,0,i,k,0); 
              }
        }   
   

   // Einige Gr��en berechnen, sie werden sp�ter ben�tigt
   for(j = 0; j < K; j++)
      {         
       if(j < K-1) 
        {     
        for(k=0; k < K-1; k++)if(k!=j){set_element(&dpdq_h,-2*parameter[q_vec[j]]*parameter[q_vec[k]]*parameter[q_vec[k]]/nenner2_h,0,j,k);}
        double temp_sum = 0; for(k = 0; k < K - 1; k++){if(k != j) temp_sum += parameter[q_vec[k]]*parameter[q_vec[k]];}
        set_element(&dpdq_h,  2*parameter[q_vec[j]] * (temp_sum + 1)/nenner2_h,0,j,j);
        }
         
        //detL_h[j] <- prod(parameter[(K*Dim + 1 + (j-1)*Dim*(Dim+1)*0.5):(K*Dim + j*Dim*(Dim+1)*0.5)][index_vec]); # Determinante
        detL_h[0] = 1;
        for(k=0; k < Dim; k++){detL_h[0] *= parameter[(int)(kdim + index_vec[k])];}
       
      }//Ende j
      
   for(k = 0; k < K - 1; k++)
      {
       for(i = 0; i <=k; i++)      
       {
        d2pdq2(i,k,parameter,q_vec,nenner_h,K, &d2pdq2_h.values[i*(K-1) + k*(K-1)*(K-1)]);  
       }
      }
      
       //zs_vals <- array(dim = c(len_L,2)) 
       //for(i in 1:len_L){zs_vals[i,] <-  zs_delta(i)}
       for(i = 0; i < len_L; i++){zs_delta(i+1, &zs_vals.values[2*i]);}
       
       
       
       double * ddet = malloc(sizeof(double)*len_L);
       for(j = 0; j < len_L; j++){ddet[j] = 0;}
       //for(ii in 1:Dim){ddet[0.5*ii*(ii+1)] <- detL[i]/parameter[(K*Dim + (i-1)*Dim*(Dim+1)*0.5)  + 0.5*ii*(ii-1) + ii]};
       for(j = 0; j < Dim; j++){ddet[(int)(0.5*(j+1)*(j+2) - 1)] = detL_h[0]/parameter[(int)(kdim + 0.5*(j+1)*(j+2) - 1)];}
       
       
       double * sum_dpdq = malloc(sizeof(double)*K);
       
      for(j = 0; j < K-1; j++)
        {
          sum_dpdq[j] = 0;
          for(i = 0; i < K-1; i++){sum_dpdq[j] += get_element(&dpdq_h,j,i);}
          
        }
       

      //jetzt schleife �ber die Datenpunkte  
      for(i = 0; i < nrow; i++) //Parallelisieren
      {
            
       mixture_derivatives_prop(Dim,
                             K, 
                             parameter, 
                             &x[i*Dim],
                             grad, 
                             hess, 
                             &zs_vals, 
                             &SigmaInv_h, 
                             detL_h,
                             ddet,
                             &dpdq_h, 
                             sum_dpdq,
                             &d2pdq2_h, 
                             nenner_h, 
                             nenner2_h, 
                             p_vec,
                             output_len, 
                             the_output); 
      }
      
      
     the_output[2] = the_output[2]; // wegen dem komischen Bug von .C interface in R, das sonst nur Teile von the_output gef�llt werden ???
     
     
     
    //#######################################
    //############# Penalisierung ############
    //#######################################
    
  if(penalize > 0)
   {
      //Kovarianzmatrix erstellen
    if(cov[0] == 0)
     {
	double cov[(int)(Dim*(Dim+1)*0.5)];
	double means[Dim];
	
	//Mittelwerte in jeder Dimension ermitteln
	for(i = 0; i < Dim ; i++)
	{
	  means[i] = 0;
	  for(j = 0; j < nrow; j++){means[i] += x[j*Dim + i];}
	  means[i] = means[i]/(double)nrow;
	}
	
	for(i = 0; i < Dim; i++)
	{
	  for(j = 0; j <= i; j++)
	  {
	    cov[k_delta(i,j)] = 0;
	    
	    for(k = 0; k < nrow; k++)
	    {
	    cov[k_delta(i,j)] += (x[k*Dim + i] - means[i]) * (x[k*Dim + j] - means[j]);
	    }
	    cov[k_delta(i,j)] =  cov[k_delta(i,j)]/((double)nrow-1);
	  }
	}
     }
     //Ende Kovarianzmatrix erstellen
       
     
     
     //#####Likelihoodwert-anpassen######
     double spur = 0;

	//Spur(Cov*Sigma_i^-1) berechnen
	for(j = 0; j < Dim; j++)
	{
	  for(k = 0; k < Dim; k++)
	  {
	    
	    for(l = 0; l < j; l++)
	    {
			 //k'te Spalte, j'te Zeile 
	     spur += get_element(&SigmaInv_h, k,j,0)*cov[k_delta(j,l)];
	    }
	    
	    for(l = j; l < Dim; l++)
	    {
	     spur += get_element(&SigmaInv_h, k,j,0)*cov[k_delta(l,j)];
	    } 
	  } 
	}
	
	the_output[output_len - 1] -=  penalize*(spur + log(1./(detL_h[0]*detL_h[0])));
	spur = 0;

     
     //Ableitungen anpassen 
	//Gradient von Li
	for(j = 0; j < len_L; j++)
	{	
	  temp_sum = 0; 
	  z = (int)get_element(&zs_vals,0,j); // Nummer der Zeile  
	  s = (int)get_element(&zs_vals,1,j); // Nummer der Spalte
	  
	  
	  for(k = s; k < z; k++){temp_sum += cov[k_delta(z,k)]*parameter[kdim + k_delta(k,s)];}
	  for(k = z; k < Dim; k++){temp_sum += cov[k_delta(k,z)]*parameter[kdim + k_delta(k,s)];}
	  
		  
	  if(z == s){temp_sum -= 1/parameter[kdim + j];}
	  the_output[K*Dim + j] -= 2*penalize*temp_sum;

	  temp_sum = 0;
	  
	  for(l = 0; l <= j; l++)
	    { 
	      int zl = (int)get_element(&zs_vals,0,l); // Nummer der Zeile  
	      int sl = (int)get_element(&zs_vals,1,l); // Nummer der Spalte
	      
	      temp_sum += (sl==s)*cov[k_delta(zl,z)];
	      if(z==s && l==j) temp_sum += 1./(parameter[kdim + j]*parameter[kdim + j]);
	      
	      the_output[len + k_delta(kdim + j, kdim + l)] -= 2*penalize*temp_sum; 
	    }
	}//Ende j
    }//Ende if penalize
    
    //####################################################################
    //################ Ende falls penalisierung aktiviert ################
    //####################################################################
     

     free(sum_dpdq);
     free(ddet);
     free_my_array(&SigmaInv_h);
     free_my_array(&dpdq_h);
     free_my_array(&d2pdq2_h); 
     free_my_array(&zs_vals);
          
  }//Ende if(prop)
}//Ende Funktion




//Funktion zum Loesen von Hx = b. Kriegt ergebnis von logli_derivatives
//LRorSOR = 0 => es wird die LR-Zerlegung benutzt, LRorSOR = 1, es wird das SOR-verfahren genutzt
int solve(double * the_input, int * CholorSOR, int * len, double * tol_eps, double * x)
{
/*
    len == K*(Dim + 0.5*Dim*(Dim+1)) + K - 1 L�NGE DES PARAMETERS
    the_input[0 : len - 1 ] Der Gradient
    the_input[len  : len + 0.5*len*(len+1) - 1] = Die Hessematrix (unterer Teil)
    Schreibe die Werte der LR-Zerlegung direkt in the_input
*/

int i,j,k;
int length = (int)(0.5*(*len)*(*len + 1));

double temp_sum = 0;
double * L = malloc(sizeof(double)*length); //L und D
//for(i = 0; i < length; i++){L[i] = 0;}

L[0] = the_input[*len];

    if(fabs(L[0]) < *tol_eps){x[0] = -12345678; return 0;}

    for(i = 1; i < *len; i++) {L[k_delta(i,0)] = the_input[*len + k_delta(i,0)]/L[0];}

    for(j = 1; j < *len - 1; j++)
    {
        for(k = 0; k < j; k++){temp_sum += L[k_delta(j,k)]*L[k_delta(j,k)]*L[k_delta(k,k)];}

        L[k_delta(j,j)] = the_input[*len + k_delta(j,j)] - temp_sum; temp_sum = 0;

        if(fabs(L[k_delta(j,j)]) < *tol_eps){x[0] = -12345678; return 0;}

        for(i= j+1; i < *len; i++)
        {
         for(k = 0; k < j; k++){temp_sum += L[k_delta(j,k)]*L[k_delta(i,k)]*L[k_delta(k,k)];}

         L[k_delta(i,j)] = (the_input[*len + k_delta(i,j)] - temp_sum)/L[k_delta(j,j)];
         temp_sum = 0;

        }
    }

    //Schritt f�r j = *len - 1
    for(k = 0; k < *len-1; k++){temp_sum +=  L[k_delta(*len-1,k)]*L[k_delta(*len-1,k)]*L[k_delta(k,k)];}
    L[k_delta(*len-1,*len-1)] = the_input[*len + k_delta(*len-1,*len-1)] - temp_sum; temp_sum = 0;

//Aufloesen LDL^Tx = b, Lx = y
  x[0] = -the_input[0]/L[k_delta(0,0)];
  
  for(i = 1; i < *len; i++)
  {
        for(j = 0; j <= i-1; j++)
        {
             temp_sum += L[k_delta(j,j)]*L[k_delta(i,j)]*x[j];
        }
                 
        x[i] = (-the_input[i] - temp_sum)/L[k_delta(i,i)]; 
        temp_sum = 0;
  }
  
  for(i = *len-2; i >=0; i--)
  {   
     for(j = i+1; j < *len; j++)
     {
           temp_sum -= L[k_delta(j,i)]*x[j];
     }    
     x[i] += temp_sum; temp_sum = 0;
  }
  
  

free(L);
return 1;
}

void newton_direction(double * parameter,
		      double * x, 
		      int * nrowp, 
		      int * Dimp, 
		      int * Kp, 
		      int * prop, 
		      double * penalize,
		      double * cov,
		      double * the_output, 
		      double * tol_eps, 
		      int * CholOrSOR, 
		      double * richtung)
{

  int * len = malloc(sizeof(int));
  int * gradp = malloc(sizeof(int)); 
  int * hessp = malloc(sizeof(int));
  *hessp = 1;
  *gradp = 1;  
  if(!*prop){*len = (int)((*Kp)*((*Dimp) + 0.5*(*Dimp)*((*Dimp)+1)) + (*Kp) - 1);}
  else{*len = (int)((*Kp)*(*Dimp) + 0.5*(*Dimp)*((*Dimp)+1) + 2*(*Kp) - 2);}

  logli_derivatives(parameter, x, nrowp, Dimp, Kp, gradp, hessp, prop, penalize, cov, the_output);
  solve(the_output, CholOrSOR, len, tol_eps, richtung); 
  
  free(len);
  free(gradp);
  free(hessp);
}



void newton_direction_par(double * parameter, 
			  double * x, 
			  int * nrowp, 
			  int * Dimp, 
			  int * Kp, 
			  int * prop,
			  double * penalize,
			  double * cov,			  
			  double * the_output, 
			  double * tol_eps, 
			  int * CholOrSOR, 
			  double * richtung)
{

  int * len = malloc(sizeof(int));
  int * gradp = malloc(sizeof(int)); 
  int * hessp = malloc(sizeof(int));
  *hessp = 1;
  *gradp = 1;  
  if(!*prop){*len = (int)((*Kp)*((*Dimp) + 0.5*(*Dimp)*((*Dimp)+1)) + (*Kp) - 1);}
  else{*len = (int)((*Kp)*(*Dimp) + 0.5*(*Dimp)*((*Dimp)+1) + 2*(*Kp) - 2);}


  logli_derivatives_par(parameter, x, nrowp, Dimp, Kp, gradp, hessp, prop, penalize, cov, the_output);
  solve(the_output, CholOrSOR, len, tol_eps, richtung); 
  
  free(len);
  free(gradp);
  free(hessp);
}
























